@@ -19,42 +19,36 @@
 #ifndef DEFS_H_INCLUDED_
 #define DEFS_H_INCLUDED_
 
-#include <algorithm>              // for sort, stable_sort
-#include <cmath>                  // for M_PI
-#include <cstdarg>                // for va_list
-#include <cstddef>                // for NULL, nullptr_t, size_t
-#include <cstdint>                // for int32_t, uint32_t
-#include <cstdio>                 // for NULL, fprintf, FILE, stdout
-#include <ctime>                  // for time_t
-#include <optional>               // for optional
-#include <utility>                // for move
+#include <cmath>                     // for M_PI
+#include <cstddef>                   // for NULL, nullptr_t, size_t
+#include <cstdint>                   // for int32_t, uint32_t
+#include <cstdio>                    // for NULL, fprintf, FILE, stdout
+#include <ctime>                     // for time_t
+#include <optional>                  // for optional
+#include <utility>                   // for move
 
-#if HAVE_LIBZ
-#include <zlib.h>                 // doesn't really belong here, but is missing elsewhere.
-#elif !ZLIB_INHIBITED
-#include "zlib.h"                 // doesn't really belong here, but is missing elsewhere.
-#endif
+#include <QByteArray>                // for QByteArray
+#include <QDate>                     // for QDate
+#include <QTime>                     // for QTime
+#include <QDateTime>                 // for QDateTime
+#include <QDebug>                    // for QDebug
+#include <QList>                     // for QList, QList<>::const_iterator, QList<>::const_reverse_iterator, QList<>::count, QList<>::reverse_iterator
+#include <QString>                   // for QString
+#include <QStringView>               // for QStringView
+#include <QTextCodec>                // for QTextCodec
+#include <QVector>                   // for QVector
+#include <Qt>                        // for CaseInsensitive
+#include <QtGlobal>                  // for QForeachContainer, qMakeForeachContainer, foreach, qint64
 
-#include <QDebug>                 // for QDebug
-#include <QList>                  // for QList, QList<>::const_reverse_iterator, QList<>::reverse_iterator
-#include <QScopedPointer>         // for QScopedPointer
-#include <QString>                // for QString
-#include <QTextCodec>             // for QTextCodec
-#include <QVector>                // for QVector
-#include <Qt>                     // for CaseInsensitive
-#include <QtGlobal>               // for foreach
-
-#include "formspec.h"             // for FormatSpecificData
-#include "inifile.h"              // for inifile_t
-#include "gbfile.h"               // doesn't really belong here, but is missing elsewhere.
-#include "session.h"              // for session_t
-#include "src/core/datetime.h"    // for DateTime
+#include "geocache.h"                // for Geocache
+#include "formspec.h"                // for FormatSpecificData
+#include "inifile.h"                 // for inifile_t
+#include "session.h"                 // for session_t
+#include "src/core/datetime.h"       // for DateTime
 
 
 #define CSTR(qstr) ((qstr).toUtf8().constData())
 #define CSTRc(qstr) ((qstr).toLatin1().constData())
-#define STRFROMUNICODE(qstr) (global_opts.codec->fromUnicode(qstr).constData())
-#define STRTOUNICODE(cstr) (global_opts.codec->toUnicode(cstr))
 
 /*
  * Amazingly, this constant is not specified in the standard...
@@ -121,39 +115,8 @@ constexpr double MPH_TO_MPS(double a) { return a * kMPSPerMPH;}
 /* knots(nautical miles/hour) to meters/second */
 constexpr double KNOTS_TO_MPS(double a)  {return a * kMPSPerKnot;}
 
-#define MILLI_TO_MICRO(t) ((t) * 1000)  /* Milliseconds to Microseconds */
-#define MICRO_TO_MILLI(t) ((t) / 1000)  /* Microseconds to Milliseconds*/
-#define CENTI_TO_MICRO(t) ((t) * 10000) /* Centiseconds to Microseconds */
-#define MICRO_TO_CENTI(t) ((t) / 10000) /* Centiseconds to Microseconds */
-
-#if __WIN32__
-#  ifndef fileno
-#    define fileno _fileno
-#  endif
-#  define strdup _strdup
-#endif
-
-#if __WIN32__
-#if !defined _CRT_SECURE_NO_DEPRECATE
-#  define _CRT_SECURE_NO_DEPRECATE 1
-#endif
-#endif
-
-/* Pathname separator character */
-#if __WIN32__
-#  define GB_PATHSEP '\\'
-#else
-#  define GB_PATHSEP '/'
-#endif
-
-/*
- *  Toss in some GNU C-specific voodoo for checking.
- */
-#if __GNUC__
-#  define PRINTFLIKE(x,y) __attribute__ ((__format__ (__printf__, (x), (y))))
-#else
-#  define PRINTFLIKE(x,y)
-#endif
+constexpr int kDatumOSGB36 = 86; // GPS_Lookup_Datum_Index("OSGB36")
+constexpr int kDautmWGS84 = 118; // GPS_Lookup_Datum_Index("WGS 84")
 
 
 /*
@@ -189,20 +152,18 @@ enum gpsdata_type {
 #define	doing_posn ((global_opts.masked_objective & POSNDATAMASK) == POSNDATAMASK)
 
 struct global_options {
-  int synthesize_shortnames;
   int debug_level;
   gpsdata_type objective;
   unsigned int	masked_objective;
   int verbose_status;	/* set by GUI wrappers for status */
-  int smart_icons;
-  int smart_names;
   inifile_t* inifile;
-  QTextCodec* codec;
+  bool synthesize_shortnames;
+  bool smart_icons;
+  bool smart_names;
 };
 
 extern global_options global_opts;
 extern const char gpsbabel_version[];
-extern time_t gpsbabel_now;	/* gpsbabel startup-time; initialized in main.c with time() */
 extern time_t gpsbabel_time;	/* gpsbabel startup-time; initialized in main.c with current_time(), ! ZERO within testo ! */
 
 enum fix_type {
@@ -214,93 +175,6 @@ enum fix_type {
   fix_pps
 };
 
-enum status_type {
-  status_unknown=0,
-  status_true,
-  status_false
-};
-
-/*
- * Extended data if waypoint happens to represent a geocache.  This is
- * totally voluntary data...
- */
-
-enum geocache_type {
-  gt_unknown = 0,
-  gt_traditional,
-  gt_multi,
-  gt_virtual,
-  gt_letterbox,
-  gt_event,
-  gt_surprise,
-  gt_webcam,
-  gt_earth,
-  gt_locationless,
-  gt_benchmark, /* Extension to Groundspeak for GSAK */
-  gt_cito,
-  gt_ape,
-  gt_mega,
-  gt_wherigo
-};
-
-enum geocache_container {
-  gc_unknown = 0,
-  gc_micro,
-  gc_other,
-  gc_regular,
-  gc_large,
-  gc_virtual,
-  gc_small
-};
-
-class utf_string
-{
-public:
-  utf_string() = default;
-  utf_string(bool html, QString str) :
-    is_html{html},
-    utfstring{std::move(str)}
-  {}
-  bool is_html{false};
-  QString utfstring;
-};
-
-class geocache_data
-{
-public:
-  geocache_data() :
-    id(0),
-    type(gt_unknown),
-    container(gc_unknown),
-    diff(0),
-    terr(0),
-    is_archived(status_unknown),
-    is_available(status_unknown),
-    is_memberonly(status_unknown),
-    has_customcoords(status_unknown),
-    placer_id(0),
-    favorite_points(0)
-  {}
-  long long id; /* The decimal cache number */
-  geocache_type type:5;
-  geocache_container container:4;
-  unsigned int diff:6; /* (multiplied by ten internally) */
-  unsigned int terr:6; /* (likewise) */
-  status_type is_archived:2;
-  status_type is_available:2;
-  status_type is_memberonly:2;
-  status_type has_customcoords:2;
-  gpsbabel::DateTime exported;
-  gpsbabel::DateTime last_found;
-  QString placer; /* Placer name */
-  int placer_id; /* Placer id */
-  QString hint; /* all these UTF8, XML entities removed, May be not HTML. */
-  utf_string desc_short;
-  utf_string desc_long;
-  int favorite_points;
-  QString personal_note;
-};
-
 class gb_color
 {
 public:
@@ -319,7 +193,7 @@ public:
   UrlLink(QString url) :
     url_(std::move(url))
   { }
-  UrlLink(const char* url) :
+  explicit UrlLink(const char* url) :
     url_(url)
   { }
   UrlLink(QString url, QString url_link_text) :
@@ -364,52 +238,14 @@ public:
   wp_flags() :
     shortname_is_synthetic(0),
     fmt_use(0),
-    temperature(0),
-    proximity(0),
-    course(0),
-    speed(0),
-    geoidheight(0),
-    depth(0),
     is_split(0),
-    new_trkseg(0) {}
+    new_trkseg(0),
+    marked_for_deletion(0) {}
   unsigned int shortname_is_synthetic:1;
   unsigned int fmt_use:2;			/* lightweight "extra data" */
-  /* "flagged fields" */
-  unsigned int temperature:1;		/* temperature field is set */
-  unsigned int proximity:1;		/* proximity field is set */
-  unsigned int course:1;			/* course field is set */
-  unsigned int speed:1;			/* speed field is set */
-  unsigned int geoidheight:1;	/* geoidheight field is set */
-  unsigned int depth:1;			/* depth field is set */
-  /* !ToDo!
-  unsigned int altitude:1;		/+ altitude field is set +/
-  ... and others
-  */
   unsigned int is_split:1;		/* the waypoint represents a split */
   unsigned int new_trkseg:1;		/* True if first in new trkseg. */
-
-};
-
-// These are dicey as they're collected on read. Subsequent filters may change
-// things, though it's unlikely to matter in practical terms.  Don't use these
-// if a false positive would be deleterious.
-#
-class global_trait
-{
-public:
-  global_trait() :
-    trait_geocaches(0),
-    trait_heartrate(0),
-    trait_cadence(0),
-    trait_power(0),
-    trait_depth(0),
-    trait_temperature(0) {}
-  unsigned int trait_geocaches:1;
-  unsigned int trait_heartrate:1;
-  unsigned int trait_cadence:1;
-  unsigned int trait_power:1;
-  unsigned int trait_depth:1;
-  unsigned int trait_temperature:1;
+  unsigned int marked_for_deletion:1;		/* True if schedulded for deletion. */
 };
 
 /*
@@ -424,11 +260,6 @@ struct bounds {
   double min_alt;	/* -unknown_alt => invalid */
 };
 
-#define WAYPT_SET(wpt,member,val) do { (wpt)->member = (val); wpt->wpt_flags.member = 1; } while (0)
-#define WAYPT_GET(wpt,member,def) ((wpt->wpt_flags.member) ? (wpt->member) : (def))
-#define WAYPT_UNSET(wpt,member) wpt->wpt_flags.member = 0
-#define WAYPT_HAS(wpt,member) (wpt->wpt_flags.member)
-
 /*
  * This is a waypoint, as stored in the GPSR.   It tries to not
  * cater to any specific model or protocol.  Anything that needs to
@@ -438,13 +269,36 @@ struct bounds {
 class Waypoint
 {
 private:
-  static geocache_data empty_gc_data;
 
-public:
+  /* Types */
+
+  class op_flags
+  {
+  public:
+    op_flags() :
+      temperature(false),
+      proximity(false),
+      course(false),
+      speed(false),
+      geoidheight(false),
+      depth(false) {}
+    bool temperature:1;		/* temperature field is set */
+    bool proximity:1;		/* proximity field is set */
+    bool course:1;			/* course field is set */
+    bool speed:1;			/* speed field is set */
+    bool geoidheight:1;	/* geoidheight field is set */
+    bool depth:1;			/* depth field is set */
+    /* !ToDo!
+    unsigned int altitude:1;		/+ altitude field is set +/
+    ... and hdop,pdop,vdop,fix,sat,heartrate,cadence,power,
+    odometer_distance
+    */
+  };
+
+  /* Data Members */
+
+  static Geocache empty_gc_data;
 
-  double latitude;		/* Degrees */
-  double longitude; 		/* Degrees */
-  double altitude; 		/* Meters. */
   double geoidheight;	/* Height (in meters) of geoid (mean sea level) above WGS84 earth ellipsoid. */
 
   /*
@@ -460,6 +314,79 @@ public:
    * The units are meters.
    */
   double proximity;
+  float course;	/* Optional: degrees true */
+  float speed;   	/* Optional: meters per second. */
+  float temperature; /* Degrees celsius */
+  op_flags opt_flags;
+
+public:
+
+  /* Special Member Functions */
+
+  Waypoint();
+  ~Waypoint();
+  Waypoint(const Waypoint& other);
+  Waypoint& operator=(const Waypoint& other);
+
+  /* Member Functions */
+
+  bool HasUrlLink() const;
+  const UrlLink& GetUrlLink() const;
+  void AddUrlLink(const UrlLink& l);
+  QString CreationTimeXML() const;
+  gpsbabel::DateTime GetCreationTime() const;
+  void SetCreationTime(const gpsbabel::DateTime& t);
+  void SetCreationTime(qint64 t, qint64 ms = 0);
+  Geocache* AllocGCData();
+  int EmptyGCData() const;
+
+// mimic std::optional interface, but use our more space
+// efficient wp_flags.
+#define GEN_WAYPT_METHODS(field) \
+  bool field##_has_value() const \
+  { \
+    return opt_flags.field; \
+  } \
+  decltype(field) field##_value() const \
+  { \
+    if (!opt_flags.field) { \
+      throw std::bad_optional_access(); \
+    } \
+    return field; \
+  } \
+  bool field##s_equal(const Waypoint& other) const \
+  { \
+    return (opt_flags.field && other.opt_flags.field && (field == other.field)) || \
+           (!opt_flags.field && !other.opt_flags.field); \
+  } \
+  decltype(field) field##_value_or(decltype(field) p) const \
+  { \
+    return (opt_flags.field)? field : p; \
+  } \
+  void set_##field(decltype(field) p) \
+  { \
+    field = p; \
+    opt_flags.field = 1; \
+  } \
+  void reset_##field() \
+  { \
+    opt_flags.field = 0; \
+  }
+
+  GEN_WAYPT_METHODS(temperature)
+  GEN_WAYPT_METHODS(proximity)
+  GEN_WAYPT_METHODS(course)
+  GEN_WAYPT_METHODS(speed)
+  GEN_WAYPT_METHODS(geoidheight)
+  GEN_WAYPT_METHODS(depth)
+
+#undef GEN_WAYPT_METHODS
+
+  /* Data Members */
+
+  double latitude;		/* Degrees */
+  double longitude; 		/* Degrees */
+  double altitude; 		/* Meters. */
 
   /* shortname is a waypoint name as stored in receiver.  It should
    * strive to be, well, short, and unique.   Enforcing length and
@@ -484,22 +411,11 @@ public:
 
   UrlList urls;
 
-  wp_flags wpt_flags;
   QString icon_descr;
 
   gpsbabel::DateTime creation_time;
 
-  /*
-   * route priority is for use by the simplify filter.  If we have
-   * some reason to believe that the route point is more important,
-   * we can give it a higher (numerically; 0 is the lowest) priority.
-   * This causes it to be removed last.
-   * This is currently used by the saroute input filter to give named
-   * waypoints (representing turns) a higher priority.
-   * This is also used by the google input filter because they were
-   * nice enough to use exactly the same priority scheme.
-   */
-  int route_priority;
+  wp_flags wpt_flags;
 
   /* Optional dilution of precision:  positional, horizontal, vertical.
    * 1 <= dop <= 50
@@ -507,36 +423,17 @@ public:
   float hdop;
   float vdop;
   float pdop;
-  float course;	/* Optional: degrees true */
-  float speed;   	/* Optional: meters per second. */
   fix_type fix;	/* Optional: 3d, 2d, etc. */
   int  sat;	/* Optional: number of sats used for fix */
 
   unsigned char heartrate; /* Beats/min. likely to get moved to fs. */
   unsigned char cadence;	 /* revolutions per minute */
   float power; /* watts, as measured by cyclists */
-  float temperature; /* Degrees celsius */
   float odometer_distance; /* Meters */
-  geocache_data* gc_data;
+  Geocache* gc_data;
   FormatSpecificDataList fs;
   const session_t* session;	/* pointer to a session struct */
   void* extra_data;	/* Extra data added by, say, a filter. */
-
-public:
-  Waypoint();
-  ~Waypoint();
-  Waypoint(const Waypoint& other);
-  Waypoint& operator=(const Waypoint& other);
-
-  bool HasUrlLink() const;
-  const UrlLink& GetUrlLink() const;
-  void AddUrlLink(const UrlLink& l);
-  QString CreationTimeXML() const;
-  gpsbabel::DateTime GetCreationTime() const;
-  void SetCreationTime(const gpsbabel::DateTime& t);
-  void SetCreationTime(qint64 t, qint64 ms = 0);
-  geocache_data* AllocGCData();
-  int EmptyGCData() const;
 };
 
 using waypt_cb = void (*)(const Waypoint*);
@@ -546,10 +443,11 @@ class WaypointList : private QList<Waypoint*>
 {
 public:
   void waypt_add(Waypoint* wpt); // a.k.a. append(), push_back()
-  void add_rte_waypt(int waypt_ct, Waypoint* wpt, bool synth, const QString& namepart, int number_digits);
+  void add_rte_waypt(int waypt_ct, Waypoint* wpt, bool synth, QStringView namepart, int number_digits);
   // FIXME: Generally it is inefficient to use an element pointer or reference to define the element to be deleted, use iterator instead,
   //        and/or implement pop_back() a.k.a. removeLast(), and/or pop_front() a.k.a. removeFirst().
   void waypt_del(Waypoint* wpt); // a.k.a. erase()
+  void del_marked_wpts();
   // FIXME: Generally it is inefficient to use an element pointer or reference to define the element to be deleted, use iterator instead,
   //        and/or implement pop_back() a.k.a. removeLast(), and/or pop_front() a.k.a. removeFirst().
   void del_rte_waypt(Waypoint* wpt);
@@ -584,19 +482,20 @@ public:
   using QList<Waypoint*>::front; // a.k.a. first()
   using QList<Waypoint*>::rbegin;
   using QList<Waypoint*>::rend;
+  using QList<Waypoint*>::size_type;
 };
 
-const global_trait* get_traits();
 void waypt_init();
 //void update_common_traits(const Waypoint* wpt);
 void waypt_add(Waypoint* wpt);
 void waypt_del(Waypoint* wpt);
+void del_marked_wpts();
 unsigned int waypt_count();
 void waypt_status_disp(int total_ct, int myct);
 //void waypt_disp_all(waypt_cb); /* template */
 //void waypt_disp_session(const session_t* se, waypt_cb cb); /* template */
 void waypt_init_bounds(bounds* bounds);
-int waypt_bounds_valid(bounds* bounds);
+bool waypt_bounds_valid(bounds* bounds);
 void waypt_add_to_bounds(bounds* bounds, const Waypoint* waypointp);
 void waypt_compute_bounds(bounds* bounds);
 Waypoint* find_waypt_by_name(const QString& name);
@@ -676,11 +575,13 @@ struct computed_trkdata {
   std::optional<double> min_spd;	/* Meters/sec */
   std::optional<double> avg_hrt;	/* Avg Heartrate */
   std::optional<double> avg_cad;	/* Avg Cadence */
+  std::optional<double> avg_pwr;	/* Avg Power */
   gpsbabel::DateTime start;		/* Min time */
   gpsbabel::DateTime end;		/* Max time */
   std::optional<int> min_hrt;			/* Min Heartrate */
   std::optional<int> max_hrt;			/* Max Heartrate */
   std::optional<int> max_cad;			/* Max Cadence */
+  std::optional<float> max_pwr;	  /* Max Power */
 };
 
 class route_head
@@ -706,6 +607,7 @@ public:
   ~route_head();
 
   int rte_waypt_ct() const {return waypoint_list.count();}		/* # waypoints in waypoint list */
+  bool rte_waypt_empty() const {return waypoint_list.empty();}
 };
 
 using route_hdr = void (*)(const route_head*);
@@ -722,14 +624,16 @@ public:
   void del_head(route_head* rte); // a.k.a. erase()
   // FIXME: Generally it is inefficient to use an element pointer or reference to define the insertion point, use iterator instead.
   void insert_head(route_head* rte, route_head* predecessor); // a.k.a. insert
-  void add_wpt(route_head* rte, Waypoint* wpt, bool synth, const QString& namepart, int number_digits);
+  void add_wpt(route_head* rte, Waypoint* wpt, bool synth, QStringView namepart, int number_digits);
   // FIXME: Generally it is inefficient to use an element pointer or reference to define the insertion point, use iterator instead.
   void del_wpt(route_head* rte, Waypoint* wpt);
+  void del_marked_wpts(route_head* rte);
   void common_disp_session(const session_t* se, route_hdr rh, route_trl rt, waypt_cb wc);
   void flush(); // a.k.a. clear()
   void copy(RouteList** dst) const;
   void restore(RouteList* src);
   void swap(RouteList& other);
+  void swap_wpts(route_head* rte, WaypointList& other);
   template <typename Compare>
   void sort(Compare cmp) {std::sort(begin(), end(), cmp);}
   template <typename T1, typename T2, typename T3>
@@ -780,10 +684,14 @@ void route_del_head(route_head* rte);
 void track_add_head(route_head* rte);
 void track_del_head(route_head* rte);
 void track_insert_head(route_head* rte, route_head* predecessor);
-void route_add_wpt(route_head* rte, Waypoint* wpt, const QString& namepart = "RPT", int number_digits = 3);
-void track_add_wpt(route_head* rte, Waypoint* wpt, const QString& namepart = "RPT", int number_digits = 3);
+void route_add_wpt(route_head* rte, Waypoint* wpt, QStringView namepart = u"RPT", int number_digits = 3);
+void track_add_wpt(route_head* rte, Waypoint* wpt, QStringView namepart = u"RPT", int number_digits = 3);
 void route_del_wpt(route_head* rte, Waypoint* wpt);
 void track_del_wpt(route_head* rte, Waypoint* wpt);
+void route_del_marked_wpts(route_head* rte);
+void track_del_marked_wpts(route_head* rte);
+void route_swap_wpts(route_head* rte, WaypointList& other);
+void track_swap_wpts(route_head* rte, WaypointList& other);
 //void route_disp(const route_head* rte, waypt_cb); /* template */
 void route_disp(const route_head* rte, std::nullptr_t /* waypt_cb */); /* override to catch nullptr */
 //void route_disp_all(route_hdr, route_trl, waypt_cb); /* template */
@@ -907,32 +815,6 @@ using ff_exit = void (*)();
 using ff_writeposn = void (*)(Waypoint*);
 using ff_readposn = Waypoint* (*)(posn_status*);
 
-geocache_type gs_mktype(const QString& t);
-geocache_container gs_mkcont(const QString& t);
-
-/*
- * All shortname functions take a shortname handle as the first arg.
- * This is an opaque pointer.  Callers must not fondle the contents of it.
- */
-// This is a crutch until the new C++ shorthandle goes in.
-
-struct mkshort_handle_imp; // forward declare, definition in mkshort.cc
-using short_handle = mkshort_handle_imp*;
-
-char* mkshort(short_handle,  const char*, bool);
-QString mkshort(short_handle,  const QString&);
-short_handle mkshort_new_handle();
-QString mkshort_from_wpt(short_handle h, const Waypoint* wpt);
-void mkshort_del_handle(short_handle* h);
-void setshort_length(short_handle, int n);
-void setshort_badchars(short_handle,  const char*);
-void setshort_goodchars(short_handle,  const char*);
-void setshort_mustupper(short_handle,  int n);
-void setshort_mustuniq(short_handle,  int n);
-void setshort_whitespace_ok(short_handle,  int n);
-void setshort_repeating_whitespace_ok(short_handle,  int n);
-void setshort_defname(short_handle, const char* s);
-
 #define ARGTYPE_UNKNOWN    0x00000000U
 #define ARGTYPE_INT        0x00000001U
 #define ARGTYPE_FLOAT      0x00000002U
@@ -970,13 +852,13 @@ void setshort_defname(short_handle, const char* s);
 #define ARG_NOMINMAX nullptr, nullptr
 
 struct arglist_t {
-  const char* argstring{nullptr};
-  char** argval{nullptr};
-  const char* helpstring{nullptr};
-  const char* defaultvalue{nullptr};
+  const QString argstring;
+  char** const argval{nullptr};
+  const QString helpstring;
+  const QString defaultvalue;
   const uint32_t argtype{ARGTYPE_UNKNOWN};
-  const char* minvalue{nullptr};    /* minimum value for numeric options */
-  const char* maxvalue{nullptr};    /* maximum value for numeric options */
+  const QString minvalue;    /* minimum value for numeric options */
+  const QString maxvalue;    /* maximum value for numeric options */
   char* argvalptr{nullptr};         /* !!! internal helper. Not used in definitions !!! */
 };
 
@@ -1033,17 +915,22 @@ struct ff_vecs_t {
   ff_write write;
   ff_exit exit;
   QVector<arglist_t>* args;
-  QString encode;
-  int fixed_encode;
   position_ops_t position_ops;
-  void* unused; /* TODO: delete this field */
 };
 
 [[noreturn]] void fatal(QDebug& msginstance);
-[[noreturn]] void fatal(const char*, ...) PRINTFLIKE(1, 2);
-void warning(const char*, ...) PRINTFLIKE(1, 2);
+// cppcheck 2.10.3 fails to assign noreturn attribute to fatal if
+// the noreturn attribute is listed before the gnu::format attribute.
+// A PR to resolve this is https://github.com/danmar/cppcheck/pull/4971,
+// but cppcheck works if the noreturn attribute follows the gnu::format
+// attribute.
+// This can have a large effect on codacy issues from cppcheck
+// nullPointerRedundantCheck, nullPointerArithmeticRedundantCheck,
+// negativeIndex, arrayIndexOutOfBoundsCond.
+[[gnu::format(printf, 1, 2)]] [[noreturn]] void fatal(const char*, ...);
+[[gnu::format(printf, 1, 2)]] void warning(const char*, ...);
 
-void printposn(double c, int is_lat);
+void printposn(double c, bool is_lat);
 
 void* xcalloc(size_t nmemb, size_t size);
 void* xmalloc(size_t size);
@@ -1051,7 +938,6 @@ void* xrealloc(void* p, size_t s);
 void xfree(const void* mem);
 char* xstrdup(const QString& s);
 char* xstrndup(const char* str, size_t sz);
-char* xstrappend(char* src, const char* newd);
 char* xstrdup(const char* s);
 
 FILE* xfopen(const char* fname, const char* type, const char* errtxt);
@@ -1059,9 +945,6 @@ FILE* xfopen(const char* fname, const char* type, const char* errtxt);
 // Thin wrapper around fopen() that supports Unicode fname on all platforms.
 FILE* ufopen(const QString& fname, const char* mode);
 
-// OS-abstracting wrapper for getting Unicode environment variables.
-QString ugetenv(const char* env_var);
-
 // FIXME: case_ignore_strcmp() and case_ignore_strncmp() should probably
 // just be replaced at the call sites.  These shims are just here to make
 // them more accommodating of QString input.
@@ -1076,61 +959,24 @@ inline int case_ignore_strncmp(const QString& s1, const QString& s2, int n)
   return s1.left(n).compare(s2.left(n), Qt::CaseInsensitive);
 }
 
-int str_match(const char* str, const char* match);
-
-char* strsub(const char* s, const char* search, const char* replace);
-char* gstrsub(const char* s, const char* search, const char* replace);
-
-void rtrim(char* s);
-char* lrtrim(char* buff);
-int xasprintf(char** strp, const char* fmt, ...) PRINTFLIKE(2, 3);
-int xasprintf(QString* strp, const char* fmt, ...) PRINTFLIKE(2, 3);
-int xasprintf(QScopedPointer<char, QScopedPointerPodDeleter>& strp, const char* fmt, ...) PRINTFLIKE(2, 3);
-int xvasprintf(char** strp, const char* fmt, va_list ap);
-char* strupper(char* src);
-char* strlower(char* src);
-signed int get_tz_offset();
-time_t mklocaltime(struct tm* t);
-time_t mkgmtime(struct tm* t);
+QDateTime make_datetime(QDate date, QTime time, bool is_localtime, bool force_utc, int utc_offset);
 bool gpsbabel_testmode();
 gpsbabel::DateTime current_time();
 QDateTime dotnet_time_to_qdatetime(long long dotnet);
-const char* get_cache_icon(const Waypoint* waypointp);
-const char* gs_get_cachetype(geocache_type t);
-const char* gs_get_container(geocache_container t);
-char* xml_entitize(const char* str);
-char* html_entitize(const QString& str);
-char* strip_html(const utf_string*);
-char* strip_nastyhtml(const QString& in);
-char* convert_human_date_format(const char* human_datef);	/* "MM,YYYY,DD" -> "%m,%Y,%d" */
-char* convert_human_time_format(const char* human_timef);	/* "HH+mm+ss"   -> "%H+%M+%S" */
-char* pretty_deg_format(double lat, double lon, char fmt, const char* sep, int html);    /* decimal ->  dd.dddd or dd mm.mmm or dd mm ss */
+long long qdatetime_to_dotnet_time(const QDateTime& dt);
+QString strip_html(const QString& utfstring);
+QString strip_nastyhtml(const QString& in);
+QString convert_human_date_format(const char* human_datef);	/* "MM,YYYY,DD" -> "%m,%Y,%d" */
+QString convert_human_time_format(const char* human_timef);	/* "HH+mm+ss"   -> "%H+%M+%S" */
+QString pretty_deg_format(double lat, double lon, char fmt, const char* sep, bool html);    /* decimal ->  dd.dddd or dd mm.mmm or dd mm ss */
 
 QString get_filename(const QString& fname);			/* extract the filename portion */
 
-/*
- * Character encoding transformations.
- */
-
-#define CET_NOT_CONVERTABLE_DEFAULT '$'
-#define CET_CHARSET_ASCII	"US-ASCII"
-#define CET_CHARSET_UTF8	"UTF-8"
-#define CET_CHARSET_HEBREW  "ISO-8859-8"
-#define CET_CHARSET_MS_ANSI	"windows-1252"
-#define CET_CHARSET_LATIN1	"ISO-8859-1"
-
 /* this lives in gpx.c */
 gpsbabel::DateTime xml_parse_time(const QString& dateTimeString);
 
 QString rot13(const QString& s);
 
-/*
- * PalmOS records like fixed-point numbers, which should be rounded
- * to deal with possible floating-point representation errors.
- */
-
-signed int si_round(double d);
-
 /*
  * Prototypes for Endianness helpers.
  */
@@ -1183,20 +1029,14 @@ enum grid_type {
 #define GRID_INDEX_MIN	grid_lat_lon_ddd
 #define GRID_INDEX_MAX	grid_swiss
 
-#define DATUM_OSGB36	86
-#define DATUM_WGS84	118
-
-/* bit manipulation functions (util.c) */
-
-char gb_getbit(const void* buf, uint32_t nr);
-void gb_setbit(void* buf, uint32_t nr);
-
 void* gb_int2ptr(int i);
 int gb_ptr2int(const void* p);
 
+QTextCodec* get_codec(const QByteArray& cs_name);
 void list_codecs();
 void list_timezones();
 QString grapheme_truncate(const QString& input, unsigned int count);
+int xstrtoi(const char* str, char** str_end, int base);
 
 /*
  *  From parse.c
@@ -1210,16 +1050,6 @@ int parse_distance(const QString& str, double* val, double scale, const char* mo
 int parse_speed(const char* str, double* val, double scale, const char* module);
 int parse_speed(const QString& str, double* val, double scale, const char* module);
 
-/*
- *  From util_crc.c
- */
-unsigned long get_crc32(const void* data, int datalen);
-
-/*
- * From nmea.c
- */
-int nmea_cksum(const char* buf);
-
 /*
  * Color helpers.
  */
@@ -1229,11 +1059,13 @@ int color_to_bbggrr(const char* cname);
  * A constant for unknown altitude.   It's tempting to just use zero
  * but that's not very nice for the folks near sea level.
  */
-#define unknown_alt 	-99999999.0
-#define unknown_color	-1
+constexpr double unknown_alt = -99999999.0;
+constexpr int unknown_color = -1;
 
-// TODO: this is a (probably temporary) shim for the C->QString conversion.
-// It's here instead of gps to avoid C/C++ linkage issues.
-int32_t GPS_Lookup_Datum_Index(const QString& n);
+#if (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
+using qhash_result_t = uint;
+#else
+using qhash_result_t = size_t;
+#endif
 
 #endif // DEFS_H_INCLUDED_
