@@ -19,36 +19,44 @@
 
  */
 
-#include <cctype>               // for isalpha, toupper
-#include <climits>              // for INT_MAX
-#include <cmath>                // for atan2, floor, sqrt
-#include <csetjmp>              // for setjmp
-#include <cstdio>               // for fprintf, fflush, snprintf, sprintf
-#include <cstdlib>              // for atoi, strtol
-#include <cstring>              // for memcpy, strlen, strncpy, strchr
-#include <ctime>                // for time_t
+#include <cassert>               // for assert
+#include <climits>               // for INT_MAX
+#include <cmath>                 // for atan2, floor, sqrt
+#include <cstdio>                // for fprintf, fflush, snprintf, snprintf
+#include <cstdlib>               // for strtol
+#include <cstring>               // for memcpy, strlen, strncpy, strchr
+#include <ctime>                 // for time_t
 
-#include <QByteArray>           // for QByteArray
-#include <QChar>                // for QChar
-#include <QString>              // for QString
-#include <Qt>                   // for CaseInsensitive
-#include <QtGlobal>             // for qPrintable, foreach
+#include <QByteArray>            // for QByteArray
+#include <QRegularExpression>    // for QRegularExpression
+#include <QString>               // for QString
+#include <QTextCodec>            // for QTextCodec
+#include <QVector>               // for QVector
+#include <Qt>                    // for CaseInsensitive
+#include <QtGlobal>              // for qPrintable, foreach
 
 #include "defs.h"
-#include "cet_util.h"           // for cet_convert_init, cet_cs_vec_utf8
-#include "format.h"             // for Format
-#include "garmin_device_xml.h"  // for gdx_get_info, gdx_info, gdx_file, gdx_jmp_buf
-#include "garmin_fs.h"          // for garmin_fs_garmin_after_read, garmin_fs_garmin_before_write
-#include "garmin_tables.h"      // for gt_find_icon_number_from_desc, PCX, gt_find_desc_from_icon_number
-#include "grtcirc.h"            // for DEG
-#include "jeeps/gps.h"
-#include "jeeps/gpsserial.h"
-#include "src/core/datetime.h"  // for DateTime
-#include "vecs.h"               // for Vecs
+#include "formspec.h"            // for FormatSpecificDataList
+#include "garmin_fs.h"           // for garmin_fs_garmin_after_read, garmin_fs_garmin_before_write
+#include "garmin_tables.h"       // for gt_find_icon_number_from_desc, PCX, gt_find_desc_from_icon_number
+#include "geocache.h"            // for Geocache, Geocache::type_t, Geocache...
+#include "grtcirc.h"             // for DEG
+#include "jeeps/gpsapp.h"        // for GPS_Set_Baud_Rate, GPS_Init, GPS_Pre...
+#include "jeeps/gpscom.h"        // for GPS_Command_Get_Lap, GPS_Command_Get...
+#include "jeeps/gpsdevice.h"     // for gpsdevh
+#include "jeeps/gpsmem.h"        // for GPS_Track_Del, GPS_Way_Del, GPS_Pvt_Del
+#include "jeeps/gpsport.h"       // for int32
+#include "jeeps/gpsprot.h"       // for gps_waypt_type, gps_category_type
+#include "jeeps/gpssend.h"       // for GPS_SWay, GPS_PWay, GPS_STrack, GPS_...
+#include "jeeps/gpsserial.h"     // for DEFAULT_BAUD
+#include "jeeps/gpsutil.h"       // for GPS_User, GPS_Enable_Diagnose, GPS_E...
+#include "src/core/datetime.h"   // for DateTime
+#include "mkshort.h"             // for MakeShort
+
 
 #define MYNAME "GARMIN"
 static const char* portname;
-static short_handle mkshort_handle;
+static MakeShort* mkshort_handle;
 static GPS_PWay* tx_waylist;
 static GPS_PWay* tx_routelist;
 static GPS_PWay* cur_tx_routelist_entry;
@@ -65,16 +73,17 @@ static char* deficon = nullptr;
 static char* category = nullptr;
 static char* categorybitsopt = nullptr;
 static char* baudopt = nullptr;
+static char* opt_codec = nullptr;
 static int baud = 0;
 static int categorybits;
-static int receiver_must_upper = 1;
-
-static Format* gpx_vec;
+static bool receiver_must_upper = true;
+static QTextCodec* codec{nullptr};
 
 #define MILITANT_VALID_WAYPT_CHARS "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
 
-/* Technically, even this is a little loose as spaces arent allowed */
+/* Technically, even this is a little loose as spaces aren't allowed */
 static const char* valid_waypt_chars = MILITANT_VALID_WAYPT_CHARS " ";
+static QRegularExpression invalid_char_re;
 
 static
 QVector<arglist_t> garmin_args = {
@@ -113,22 +122,45 @@ QVector<arglist_t> garmin_args = {
   },
   {
     "baud", &baudopt, "Speed in bits per second of serial port (baud=9600)",
-    nullptr, ARGTYPE_INT, ARG_NOMINMAX, nullptr },
+    nullptr, ARGTYPE_INT, ARG_NOMINMAX, nullptr
+  },
+    {
+      "codec", &opt_codec, "override codec to use for device",
+      nullptr, ARGTYPE_STRING, ARG_NOMINMAX, nullptr
+    },
 
 };
 
 static const char* d103_symbol_from_icon_number(unsigned int n);
 static int d103_icon_number_from_symbol(const QString& s);
+static void garmin_fs_garmin_after_read(GPS_PWay way, Waypoint* wpt, int protoid);
+static void garmin_fs_garmin_before_write(const Waypoint* wpt, GPS_PWay way, int protoid);
 
+static QByteArray str_from_unicode(const QString& qstr)
+{
+  return codec->fromUnicode(qstr);
+}
+static QString str_to_unicode(const QByteArray& cstr)
+{
+  return codec->toUnicode(cstr);
+}
+
+static void
+write_char_string(char* dest, const char* source, size_t destsize)
+{
+  // we zero fill and always terminate within the dest buffer.
+  strncpy(dest, source, destsize - 1);
+  dest[destsize-1] = 0;
+}
 
 static void
 rw_init(const QString& fname)
 {
-  receiver_must_upper = 1;
-  const char* receiver_charset = nullptr;
+  receiver_must_upper = true;
+  const char* receiver_charset = "US-ASCII";
 
   if (!mkshort_handle) {
-    mkshort_handle = mkshort_new_handle();
+    mkshort_handle = new MakeShort;
   }
 
   if (global_opts.debug_level > 0)  {
@@ -152,14 +184,14 @@ rw_init(const QString& fname)
   if (baudopt) {
     baud = strtol(baudopt, nullptr, 0);
     switch (baud) {
-      case 9600:
-      case 19200:
-      case 38400:
-      case 57600:
-      case 115200:
-        break;
-      default:
-        fatal("Baud rate %d is not supported\n", baud);
+    case 9600:
+    case 19200:
+    case 38400:
+    case 57600:
+    case 115200:
+      break;
+    default:
+      fatal("Baud rate %d is not supported\n", baud);
     }
   }
 
@@ -168,7 +200,7 @@ rw_init(const QString& fname)
   }
 
   /*
-   * THis is Gross. The B&W Vista sometimes sets its time decades into
+   * This is Gross. The B&W Vista sometimes sets its time decades into
    * the future with no way to reset it.  This apparently can "cure"
    * an affected unit.
    */
@@ -187,7 +219,7 @@ rw_init(const QString& fname)
   }
 
   /*
-   * Grope the unit we're talking to to set setshort_length to
+   * Grope the unit we're talking to to set set_length to
    * 	20 for  the V,
    * 	10 for Street Pilot, (old) Rhino, 76
    * 	6 for the III, 12, emap, and etrex
@@ -228,9 +260,8 @@ rw_init(const QString& fname)
     case 696: 	/* eTrex HC */
     case 574: 	/* Geko 201 */
       receiver_short_length = 6;
-      valid_waypt_chars =
-        MILITANT_VALID_WAYPT_CHARS " +-";
-      setshort_badchars(mkshort_handle, "\"$.,'!");
+      valid_waypt_chars = MILITANT_VALID_WAYPT_CHARS " +-";
+      mkshort_handle->set_badchars("\"$.,'!");
       break;
 
     case 155:	/* Garmin V */
@@ -240,7 +271,7 @@ rw_init(const QString& fname)
       break;
     case 382: 	/* C320 */
       receiver_short_length = 30;
-      receiver_must_upper = 0;
+      receiver_must_upper = false;
       break;
     case 292: /* (60|76)C[S]x series */
     case 421: /* Vista|Legend Cx */
@@ -250,29 +281,29 @@ rw_init(const QString& fname)
     case 957: /* Legend HC */
       receiver_short_length = 14;
       snwhiteopt = xstrdup("1");
-      receiver_must_upper = 0;
+      receiver_must_upper = false;
       /* This might be 8859-1 */
-      receiver_charset = CET_CHARSET_MS_ANSI;
+      receiver_charset = "windows-1252";
       break;
     case 291: /* GPSMAP 60CS, probably others */
     case 1095: /* GPS 72H */
       receiver_short_length = 10;
       valid_waypt_chars = MILITANT_VALID_WAYPT_CHARS " +-";
-      setshort_badchars(mkshort_handle, "\"$.,'!");
+      mkshort_handle->set_badchars("\"$.,'!");
       break;
     case 231: /* Quest */
     case 463: /* Quest 2 */
-      receiver_must_upper = 0;
+      receiver_must_upper = false;
       receiver_short_length = 30;
-      receiver_charset = CET_CHARSET_MS_ANSI;
+      receiver_charset = "windows-1252";
       break;
     case 577: // Rino 530HCx Version 2.50
-      receiver_must_upper = 0;
+      receiver_must_upper = false;
       receiver_short_length = 14;
       break;
     case 429: // Streetpilot i3
-      receiver_must_upper = 0;
-      receiver_charset = CET_CHARSET_MS_ANSI;
+      receiver_must_upper = false;
+      receiver_charset = "windows-1252";
       receiver_short_length = 30;
       break;
     case 484: // Forerunner 305
@@ -302,51 +333,61 @@ rw_init(const QString& fname)
    * If the user provided a short_length, override the calculated value.
    */
   if (snlen) {
-    setshort_length(mkshort_handle, atoi(snlen));
+    mkshort_handle->set_length(xstrtoi(snlen, nullptr, 10));
   } else {
-    setshort_length(mkshort_handle, receiver_short_length);
+    mkshort_handle->set_length(receiver_short_length);
   }
 
   if (snwhiteopt) {
-    setshort_whitespace_ok(mkshort_handle, atoi(snwhiteopt));
+    mkshort_handle->set_whitespace_ok(xstrtoi(snwhiteopt, nullptr, 10));
   }
 
   /*
-   * Until Garmins documents how to determine valid character space
+   * Until Garmin documents how to determine valid character space
    * for the new models, we just release this safety check manually.
    */
   if (receiver_must_upper) {
-    setshort_goodchars(mkshort_handle, valid_waypt_chars);
+    mkshort_handle->set_goodchars(valid_waypt_chars);
   } else {
-    setshort_badchars(mkshort_handle, "");
+    mkshort_handle->set_badchars("");
   }
 
-  setshort_mustupper(mkshort_handle, receiver_must_upper);
+  mkshort_handle->set_mustupper(receiver_must_upper);
 
   /*
    * This used to mean something when we used cet, but these days this
-   * format either use implicit QString conversions (utf8) which is
-   * likely a bug, or we have hard coded QString::fromLatin1 or CSTRc.
-   * So all the above detection of receiver_charset is for naught.
-   * But perhaps we will use an appropriate codec based on receiver_charset
-   * someday.
+   * format either uses implicit QString conversions (utf8),
+   * or we have hard coded QString::fromLatin1, CSTRc, or CSTR.  These
+   * are likely bugs.
+   * However, this is still used for garmin_fs_garmin_after_read,
+   * garmin_fs_garmin_before_write.
    */
-  if (receiver_charset) {
-    cet_convert_init(receiver_charset, 1);
+  if (opt_codec != nullptr) {
+    // override expected codec with user supplied choice.
+    receiver_charset = opt_codec;
   }
+  codec = get_codec(receiver_charset);
+  if (global_opts.verbose_status) {
+    fprintf(stdout, "receiver charset detected as %s.\r\n", receiver_charset);
+  }
+
+  /*
+   * Beware, valid_waypt_chars shouldn't contain any character class metacharacters,
+   * i.e. '\', '^', '-', '[', or ']'
+   */
+  assert(!QString(valid_waypt_chars).contains('\\'));
+  assert(!QString(valid_waypt_chars).contains('^'));
+  assert(!QString(valid_waypt_chars).contains('-'));
+  assert(!QString(valid_waypt_chars).contains('['));
+  assert(!QString(valid_waypt_chars).contains(']'));
+  invalid_char_re = QRegularExpression(QStringLiteral("[^%1]").arg(valid_waypt_chars));
+  assert(invalid_char_re.isValid());
 }
 
 static void
 rd_init(const QString& fname)
 {
-  if (setjmp(gdx_jmp_buf)) {
-    const gdx_info* gi = gdx_get_info();
-    gpx_vec = Vecs::Instance().find_vec("gpx");
-    gpx_vec->rd_init(gi->from_device.canon);
-  } else {
-    gpx_vec = nullptr;
-    rw_init(fname);
-  }
+  rw_init(fname);
 }
 
 static void
@@ -358,16 +399,15 @@ rw_deinit()
     }
   }
 
-  if (mkshort_handle) {
-    mkshort_del_handle(&mkshort_handle);
-  }
+  delete mkshort_handle;
+  mkshort_handle = nullptr;
 
   xfree(portname);
   portname = nullptr;
 }
 
 static int
-waypt_read_cb(int total_ct, GPS_PWay* )
+waypt_read_cb(int total_ct, GPS_PWay*)
 {
   if (global_opts.verbose_status) {
     static int i;
@@ -402,8 +442,8 @@ waypt_read()
   for (int i = 0; i < n; i++) {
     auto* wpt_tmp = new Waypoint;
 
-    wpt_tmp->shortname = QString::fromLatin1(way[i]->ident);
-    wpt_tmp->description = QString::fromLatin1(way[i]->cmnt);
+    wpt_tmp->shortname = str_to_unicode(way[i]->ident);
+    wpt_tmp->description = str_to_unicode(way[i]->cmnt);
     wpt_tmp->shortname = wpt_tmp->shortname.simplified();
     wpt_tmp->description = wpt_tmp->description.simplified();
     wpt_tmp->longitude = way[i]->lon;
@@ -417,8 +457,8 @@ waypt_read()
     }
     /*
      * If a unit doesn't store altitude info (i.e. a D103)
-     * gpsmem will default the alt to INT_MAX.   Other units
-     * (I can't recall if it was the V (D109) hor the Vista (D108)
+     * gpsmem will default the alt to INT_MAX.  Other units
+     * (I can't recall if it was the V (D109) or the Vista (D108)
      * return INT_MAX+1, contrary to the Garmin protocol doc which
      * says they should report 1.0e25.   So we'll try to trap
      * all the cases here.     Yes, libjeeps should probably
@@ -444,7 +484,7 @@ waypt_read()
   }
 }
 
-static int lap_read_nop_cb(int, struct GPS_SWay**)
+static int lap_read_nop_cb(int, GPS_SWay**)
 {
   return 0;
 }
@@ -516,7 +556,7 @@ track_read()
     if (trk_head == nullptr || array[i]->ishdr) {
       trk_head = new route_head;
       trk_head->rte_num = trk_num;
-      trk_head->rte_name = QString::fromLatin1(trk_name);
+      trk_head->rte_name = str_to_unicode(trk_name);
       trk_num++;
       track_add_head(trk_head);
     }
@@ -538,7 +578,7 @@ track_read()
     wpt->altitude = array[i]->alt;
     wpt->heartrate = array[i]->heartrate;
     wpt->cadence = array[i]->cadence;
-    wpt->shortname = array[i]->trk_ident;
+    wpt->shortname = str_to_unicode(array[i]->trk_ident);
     wpt->SetCreationTime(array[i]->Time);
     wpt->wpt_flags.is_split = checkWayPointIsAtSplit(wpt, laps,
                               nlaps);
@@ -546,10 +586,10 @@ track_read()
     next_is_new_trkseg = 0;
 
     if (array[i]->dpth < 1.0e25f) {
-      WAYPT_SET(wpt, depth, array[i]->dpth);
+      wpt->set_depth(array[i]->dpth);
     }
     if (array[i]->temperature_populated) {
-      WAYPT_SET(wpt, temperature, array[i]->temperature);
+      wpt->set_temperature(array[i]->temperature);
     }
 
     track_add_wpt(trk_head, wpt);
@@ -595,7 +635,7 @@ route_read()
       rte_head = new route_head;
       route_add_head(rte_head);
       if (csrc) {
-        rte_head->rte_name = QString::fromLatin1(csrc);
+        rte_head->rte_name = str_to_unicode(csrc);
       }
     } else {
       if (array[i]->islink)  {
@@ -604,7 +644,7 @@ route_read()
         auto* wpt_tmp = new Waypoint;
         wpt_tmp->latitude = array[i]->lat;
         wpt_tmp->longitude = array[i]->lon;
-        wpt_tmp->shortname = array[i]->ident;
+        wpt_tmp->shortname = str_to_unicode(array[i]->ident);
         route_add_wpt(rte_head, wpt_tmp);
       }
     }
@@ -615,97 +655,6 @@ route_read()
 
 }
 
-#if 0
-static
-void
-lap_read_as_track(void)
-{
-  int32 ntracks;
-  GPS_PLap* array;
-  route_head* trk_head = NULL;
-  int trk_num = 0;
-  int index;
-  int i;
-  char tbuf[128];
-
-  ntracks = GPS_Command_Get_Lap(portname, &array, waypt_read_cb);
-  if (ntracks <= 0) {
-    return;
-  }
-  for (i = 0; i < ntracks; i++) {
-    Waypoint* wpt;
-    if (array[i]->index == -1) {
-      index=i;
-    } else {
-      index=array[i]->index;
-      index=i;
-    }
-
-    if ((trk_head == NULL) || (i == 0) ||
-        /* D906 - last track:index is the track index */
-        (array[i]->index == -1 && array[i]->track_index != 255) ||
-        /* D10xx - no real separator, use begin/end time to guess */
-        (abs(array[i-1]->start_time + array[i]->total_time/100-array[i]->start_time) > 2)
-       ) {
-      static struct tm* stmp;
-      stmp = gmtime(&array[i]->start_time);
-      trk_head = new route_head;
-      /*For D906, we would like to use the track_index in the last packet instead...*/
-      trk_head->rte_num = ++trk_num;
-      strftime(tbuf, 32, "%Y-%m-%dT%H:%M:%SZ", stmp);
-      trk_head->rte_name = tbuf;
-      track_add_head(trk_head);
-
-      wpt = new Waypoint;
-
-      wpt->longitude = array[i]->begin_lon;
-      wpt->latitude = array[i]->begin_lat;
-      wpt->heartrate = array[i]->avg_heart_rate;
-      wpt->cadence = array[i]->avg_cadence;
-      wpt->speed = array[i]->max_speed;
-      wpt->creation_time = array[i]->start_time;
-      wpt->microseconds = 0;
-
-      sprintf(tbuf, "#%d-0", index);
-      wpt->shortname = tbuf;
-      sprintf(tbuf, "D:%f Cal:%d MS:%f AH:%d MH:%d AC:%d I:%d T:%d",
-              array[i]->total_distance, array[i]->calories, array[i]->max_speed, array[i]->avg_heart_rate,
-              array[i]->max_heart_rate, array[i]->avg_cadence, array[i]->intensity, array[i]->trigger_method);
-      wpt->description = tbuf;
-      track_add_wpt(trk_head, wpt);
-    }
-    /*Allow even if no correct location, no skip if invalid */
-    /*		if (array[i]->no_latlon) {
-    *			continue;
-    *		}
-    */
-    wpt = new Waypoint;
-
-    wpt->longitude = array[i]->end_lon;
-    wpt->latitude = array[i]->end_lat;
-    wpt->heartrate = array[i]->avg_heart_rate;
-    wpt->cadence = array[i]->avg_cadence;
-    wpt->speed = array[i]->max_speed;
-    wpt->creation_time = array[i]->start_time + array[i]->total_time/100;
-    wpt->microseconds = 10000*(array[i]->total_time % 100);
-    /*Add fields with no mapping in the description */
-    sprintf(tbuf, "#%d", index);
-    wpt->shortname = tbuf;
-    sprintf(tbuf, "D:%f Cal:%d MS:%f AH:%d MH:%d AC:%d I:%d T:%d (%f,%f)",
-            array[i]->total_distance, array[i]->calories, array[i]->max_speed, array[i]->avg_heart_rate,
-            array[i]->max_heart_rate, array[i]->avg_cadence, array[i]->intensity, array[i]->trigger_method,
-            array[i]->begin_lon, array[i]->begin_lat);
-    wpt->description = tbuf;
-
-    track_add_wpt(trk_head, wpt);
-  }
-  while (ntracks) {
-    GPS_Lap_Del(&array[--ntracks]);
-  }
-  xfree(array);
-}
-#endif
-
 /*
  * Rather than propagate Garmin-specific data types outside of the Garmin
  * code, we convert the PVT (position/velocity/time) data from the receiver
@@ -717,13 +666,11 @@ pvt2wpt(GPS_PPvt_Data pvt, Waypoint* wpt)
   wpt->altitude = pvt->alt;
   wpt->latitude = pvt->lat;
   wpt->longitude = pvt->lon;
-  WAYPT_SET(wpt,course,1);
-  WAYPT_SET(wpt,speed,1);
 
-  wpt->course = 180 + DEG(atan2(-pvt->east, -pvt->north));
+  wpt->set_course(180 + DEG(std::atan2(-pvt->east, -pvt->north)));
 
   /* velocity in m/s */
-  WAYPT_SET(wpt,speed, sqrt(pvt->north*pvt->north + pvt->east*pvt->east));
+  wpt->set_speed(std::sqrt(pvt->north*pvt->north + pvt->east*pvt->east));
   // wpt->vs = pvt->up;
 
   /*
@@ -734,8 +681,8 @@ pvt2wpt(GPS_PPvt_Data pvt, Waypoint* wpt)
    *    reference clocks.
    */
   double wptime = 631065600.0 + pvt->wn_days * 86400.0  +
-    pvt->tow
-    - pvt->leap_scnds;
+                  pvt->tow
+                  - pvt->leap_scnds;
   double wptimes = floor(wptime);
   wpt->SetCreationTime(wptimes, 1000000.0 * (wptime - wptimes));
 
@@ -818,11 +765,6 @@ pvt_read(posn_status* posn_status)
 static void
 data_read()
 {
-  if (gpx_vec) {
-    gpx_vec->read();
-    return;
-  }
-
   if (poweroff) {
     return;
   }
@@ -889,25 +831,25 @@ static const char*
 get_gc_info(const Waypoint* wpt)
 {
   if (global_opts.smart_names) {
-    if (wpt->gc_data->type == gt_virtual) {
+    if (wpt->gc_data->type == Geocache::type_t::gt_virtual) {
       return  "V ";
     }
-    if (wpt->gc_data->type == gt_unknown) {
+    if (wpt->gc_data->type == Geocache::type_t::gt_unknown) {
       return  "? ";
     }
-    if (wpt->gc_data->type == gt_multi) {
+    if (wpt->gc_data->type == Geocache::type_t::gt_multi) {
       return  "Mlt ";
     }
-    if (wpt->gc_data->type == gt_earth) {
+    if (wpt->gc_data->type == Geocache::type_t::gt_earth) {
       return  "EC ";
     }
-    if (wpt->gc_data->type == gt_event) {
+    if (wpt->gc_data->type == Geocache::type_t::gt_event) {
       return  "Ev ";
     }
-    if (wpt->gc_data->container == gc_micro) {
+    if (wpt->gc_data->container == Geocache::container_t::gc_micro) {
       return  "M ";
     }
-    if (wpt->gc_data->container == gc_small) {
+    if (wpt->gc_data->container == Geocache::container_t::gc_small) {
       return  "S ";
     }
   }
@@ -922,7 +864,7 @@ waypoint_prepare()
   extern WaypointList* global_waypoint_list;
   int icon;
 
-  tx_waylist = (struct GPS_SWay**) xcalloc(n,sizeof(*tx_waylist));
+  tx_waylist = (GPS_SWay**) xcalloc(n,sizeof(*tx_waylist));
 
   for (i = 0; i < n; i++) {
     tx_waylist[i] = sane_GPS_Way_New();
@@ -946,35 +888,37 @@ waypoint_prepare()
      * mkshort will do collision detection and namespace
      * cleaning
      */
-    char* ident = mkshort(mkshort_handle,
-                           global_opts.synthesize_shortnames ? CSTRc(src) :
-                             CSTRc(wpt->shortname), false);
+    QByteArray ident = mkshort_handle->mkshort(
+                               global_opts.synthesize_shortnames ?
+                               str_from_unicode(src) :
+                               str_from_unicode(wpt->shortname),
+                               false);
     /* Should not be a strcpy as 'ident' isn't really a C string,
      * but rather a garmin "fixed length" buffer that's padded
      * to the end with spaces.  So this is NOT (strlen+1).
      */
-    memcpy(tx_waylist[i]->ident, ident, strlen(ident));
-
-    if (global_opts.synthesize_shortnames) {
-      xfree(ident);
-    }
-    tx_waylist[i]->ident[sizeof(tx_waylist[i]->ident)-1] = 0;
+    write_char_string(tx_waylist[i]->ident, ident.constData(), sizeof(tx_waylist[i]->ident));
 
     // If we were explicitly given a comment from GPX, use that.
     //  This logic really is horrible and needs to be untangled.
     if (!wpt->description.isEmpty() &&
         global_opts.smart_names && !wpt->gc_data->diff) {
-      memcpy(tx_waylist[i]->cmnt, CSTRc(wpt->description), strlen(CSTRc(wpt->description)));
+      write_char_string(tx_waylist[i]->cmnt,
+                        str_from_unicode(wpt->description).constData(),
+                        sizeof(tx_waylist[i]->cmnt));
     } else {
       if (global_opts.smart_names &&
           wpt->gc_data->diff && wpt->gc_data->terr) {
+        static_assert(sizeof(obuf) >= sizeof(tx_waylist[i]->cmnt));
         snprintf(obuf, sizeof(obuf), "%s%u/%u %s",
                  get_gc_info(wpt),
                  wpt->gc_data->diff, wpt->gc_data->terr,
-                 CSTRc(src));
-        memcpy(tx_waylist[i]->cmnt, obuf, strlen(obuf));
+                 str_from_unicode(src).constData());
+        write_char_string(tx_waylist[i]->cmnt, obuf, sizeof(tx_waylist[i]->cmnt));
       } else  {
-        memcpy(tx_waylist[i]->cmnt, CSTRc(src), strlen(CSTRc(src)));
+        write_char_string(tx_waylist[i]->cmnt,
+                          str_from_unicode(src).constData(),
+                          sizeof(tx_waylist[i]->cmnt));
       }
     }
 
@@ -986,8 +930,8 @@ waypoint_prepare()
     if (deficon) {
       icon = gt_find_icon_number_from_desc(deficon, PCX);
     } else {
-      if (get_cache_icon(wpt)) {
-        icon = gt_find_icon_number_from_desc(get_cache_icon(wpt), PCX);
+      if (!wpt->gc_data->get_icon().isEmpty()) {
+        icon = gt_find_icon_number_from_desc(wpt->gc_data->get_icon(), PCX);
       } else {
         icon = gt_find_icon_number_from_desc(wpt->icon_descr, PCX);
       }
@@ -1012,7 +956,7 @@ waypoint_prepare()
       tx_waylist[i]->time_populated = 1;
     }
     if (category) {
-      tx_waylist[i]->category = 1 << (atoi(category) - 1);
+      tx_waylist[i]->category = 1 << (xstrtoi(category, nullptr, 10) - 1);
     }
     if (categorybits) {
       tx_waylist[i]->category = categorybits;
@@ -1049,9 +993,9 @@ route_hdr_pr(const route_head* rte)
   (*cur_tx_routelist_entry)->rte_num = rte->rte_num;
   (*cur_tx_routelist_entry)->isrte = 1;
   if (!rte->rte_name.isEmpty()) {
-    strncpy((*cur_tx_routelist_entry)->rte_ident, CSTRc(rte->rte_name),
-            sizeof((*cur_tx_routelist_entry)->rte_ident) - 1);
-    (*cur_tx_routelist_entry)->rte_ident[sizeof((*cur_tx_routelist_entry)->rte_ident) - 1] = 0;
+    write_char_string((*cur_tx_routelist_entry)->rte_ident,
+                      str_from_unicode(rte->rte_name).constData(),
+                      sizeof((*cur_tx_routelist_entry)->rte_ident));
   }
 }
 
@@ -1086,23 +1030,24 @@ route_waypt_pr(const Waypoint* wpt)
     rte->alt = 0;
   }
 
-  char* d = rte->ident;
-  for (auto idx : wpt->shortname) {
-    int c = idx.toLatin1();
-    if (receiver_must_upper && isalpha(c)) {
-      c = toupper(c);
-    }
-    if (strchr(valid_waypt_chars, c)) {
-      *d++ = c;
-    }
+  QString cleanname = wpt->shortname;
+  /*
+   * Until Garmin documents how to determine valid character space
+   * for the new models, we just release this safety check manually.
+   */
+  if (receiver_must_upper) {
+    cleanname = cleanname.toUpper().remove(invalid_char_re);
   }
+  write_char_string(rte->ident,
+                    str_from_unicode(cleanname).constData(),
+                    sizeof(rte->ident));
 
-  rte->ident[sizeof(rte->ident)-1] = 0;
   if (wpt->description.isEmpty()) {
     rte->cmnt[0] = 0;
   } else {
-    strncpy(rte->cmnt, CSTR(wpt->description), sizeof(rte->cmnt) - 1);
-    rte->cmnt[sizeof(rte->cmnt)-1] = 0;
+    write_char_string(rte->cmnt,
+                      str_from_unicode(wpt->description).constData(),
+                      sizeof(rte->cmnt));
   }
   cur_tx_routelist_entry++;
 }
@@ -1112,7 +1057,7 @@ route_write()
 {
   int n = 2 * route_waypt_count(); /* Doubled for the islink crap. */
 
-  tx_routelist = (struct GPS_SWay**) xcalloc(n,sizeof(GPS_PWay));
+  tx_routelist = (GPS_SWay**) xcalloc(n,sizeof(GPS_PWay));
   cur_tx_routelist_entry = tx_routelist;
 
   for (int i = 0; i < n; i++) {
@@ -1128,10 +1073,11 @@ track_hdr_pr(const route_head* trk_head)
 {
   (*cur_tx_tracklist_entry)->ishdr = true;
   if (!trk_head->rte_name.isEmpty()) {
-    strncpy((*cur_tx_tracklist_entry)->trk_ident, CSTRc(trk_head->rte_name), sizeof((*cur_tx_tracklist_entry)->trk_ident) - 1);
-    (*cur_tx_tracklist_entry)->trk_ident[sizeof((*cur_tx_tracklist_entry)->trk_ident)-1] = 0;
+    write_char_string((*cur_tx_tracklist_entry)->trk_ident,
+                      str_from_unicode(trk_head->rte_name).constData(),
+                      sizeof((*cur_tx_tracklist_entry)->trk_ident));
   } else {
-    sprintf((*cur_tx_tracklist_entry)->trk_ident, "TRACK%02d", my_track_count);
+    snprintf((*cur_tx_tracklist_entry)->trk_ident, sizeof((*cur_tx_tracklist_entry)->trk_ident), "TRACK%02d", my_track_count);
   }
   cur_tx_tracklist_entry++;
   my_track_count++;
@@ -1145,8 +1091,9 @@ track_waypt_pr(const Waypoint* wpt)
   (*cur_tx_tracklist_entry)->alt = (wpt->altitude != unknown_alt) ? wpt->altitude : 1e25;
   (*cur_tx_tracklist_entry)->Time = wpt->GetCreationTime().toTime_t();
   if (!wpt->shortname.isEmpty()) {
-    strncpy((*cur_tx_tracklist_entry)->trk_ident, CSTRc(wpt->shortname), sizeof((*cur_tx_tracklist_entry)->trk_ident) - 1);
-    (*cur_tx_tracklist_entry)->trk_ident[sizeof((*cur_tx_tracklist_entry)->trk_ident)-1] = 0;
+    write_char_string((*cur_tx_tracklist_entry)->trk_ident,
+                      str_from_unicode(wpt->shortname).constData(),
+                      sizeof((*cur_tx_tracklist_entry)->trk_ident));
   }
   (*cur_tx_tracklist_entry)->tnew = wpt->wpt_flags.new_trkseg;
   cur_tx_tracklist_entry++;
@@ -1157,7 +1104,7 @@ track_prepare()
 {
   int32 n = track_waypt_count() + track_count();
 
-  tx_tracklist = (struct GPS_STrack**) xcalloc(n, sizeof(GPS_PTrack));
+  tx_tracklist = (GPS_STrack**) xcalloc(n, sizeof(GPS_PTrack));
   cur_tx_tracklist_entry = tx_tracklist;
   for (int i = 0; i < n; i++) {
     tx_tracklist[i] = GPS_Track_New();
@@ -1244,9 +1191,7 @@ ff_vecs_t garmin_vecs = {
   data_write,
   nullptr,
   &garmin_args,
-  CET_CHARSET_ASCII, 0,
-  { pvt_init, pvt_read, rw_deinit, nullptr, nullptr, nullptr },
-  nullptr
+  { pvt_init, pvt_read, rw_deinit, nullptr, nullptr, nullptr }
 };
 
 static const char* d103_icons[16] = {
@@ -1292,3 +1237,73 @@ d103_icon_number_from_symbol(const QString& s)
   }
   return 0;
 }
+
+static void
+garmin_fs_garmin_after_read(const GPS_PWay way, Waypoint* wpt, const int protoid)
+{
+  garmin_fs_t* gmsd = garmin_fs_alloc(protoid);
+  wpt->fs.FsChainAdd(gmsd);
+
+  /* nothing happens until gmsd is allocated some lines above */
+
+  /* !!! class needs protocol specific conversion !!! (ToDo)
+  garmin_fs_t::set_wpt_class(gmsd, way[i]->wpt_class);
+  */
+  /* flagged data fields */
+  garmin_fs_t::set_display(gmsd, gt_switch_display_mode_value(way->dspl, gps_waypt_type, 1));
+  if (way->category != 0) {
+    garmin_fs_t::set_category(gmsd, way->category);
+  }
+  if (way->dst < 1.0e25f) {
+    wpt->set_proximity(way->dst);
+  }
+  if (way->temperature_populated) {
+    wpt->set_temperature(way->temperature);
+  }
+  if (way->dpth < 1.0e25f) {
+    wpt->set_depth(way->dpth);
+  }
+  /* will copy until a null character or the end of the fixed length way field is reached, whichever comes first. */
+  garmin_fs_t::set_cc(gmsd, str_to_unicode(QByteArray(way->cc, qstrnlen(way->cc, sizeof(way->cc)))));
+  garmin_fs_t::set_city(gmsd, str_to_unicode(QByteArray(way->city, qstrnlen(way->city, sizeof(way->city)))));
+  garmin_fs_t::set_state(gmsd, str_to_unicode(QByteArray(way->state, qstrnlen(way->state, sizeof(way->state)))));
+  garmin_fs_t::set_facility(gmsd, str_to_unicode(QByteArray(way->facility, qstrnlen(way->facility, sizeof(way->facility)))));
+  garmin_fs_t::set_cross_road(gmsd, str_to_unicode(QByteArray(way->cross_road, qstrnlen(way->cross_road, sizeof(way->cross_road)))));
+  garmin_fs_t::set_addr(gmsd, str_to_unicode(QByteArray(way->addr, qstrnlen(way->addr, sizeof(way->addr)))));
+}
+
+static void
+garmin_fs_garmin_before_write(const Waypoint* wpt, GPS_PWay way, const int protoid)
+{
+  garmin_fs_t* gmsd = garmin_fs_t::find(wpt);
+
+  (void)protoid; // unused for now.
+
+  if (gmsd == nullptr) {
+    return;
+  }
+
+  /* ToDo: protocol specific conversion of class
+  way[i]->wpt_class = garmin_fs_t::get_wpt_class(gmsd, way[i]->wpt_class);
+  	*/
+  way->dspl = gt_switch_display_mode_value(
+                garmin_fs_t::get_display(gmsd, way->dspl), gps_waypt_type, 0);
+  way->category = garmin_fs_t::get_category(gmsd, way->category);
+  if (wpt->depth_has_value()) {
+    way->dpth = wpt->depth_value();
+  }
+  if (wpt->proximity_has_value()) {
+    way->dst = wpt->proximity_value();
+  }
+  if (wpt->temperature_has_value()) {
+    way->temperature = wpt->temperature_value();
+  }
+
+  /* destination may not be null terminated, but we will fill with nulls if necessary */
+  strncpy(way->cc, str_from_unicode(garmin_fs_t::get_cc(gmsd, nullptr)).constData(), sizeof(way->cc));
+  strncpy(way->city, str_from_unicode(garmin_fs_t::get_city(gmsd, nullptr)).constData(), sizeof(way->city));
+  strncpy(way->state, str_from_unicode(garmin_fs_t::get_state(gmsd, nullptr)).constData(), sizeof(way->state));
+  strncpy(way->facility, str_from_unicode(garmin_fs_t::get_facility(gmsd, nullptr)).constData(), sizeof(way->facility));
+  strncpy(way->cross_road, str_from_unicode(garmin_fs_t::get_cross_road(gmsd, nullptr)).constData(), sizeof(way->cross_road));
+  strncpy(way->addr, str_from_unicode(garmin_fs_t::get_addr(gmsd, nullptr)).constData(), sizeof(way->addr));
+}
