@@ -3,6 +3,7 @@
     Support for MapSource Text Export (Tab delimited) files.
 
     Copyright (C) 2006 Olaf Klein, o.b.klein@gpsbabel.org
+    Copyright (C) 2004-2022 Robert Lipe, robertlipe+source@gpsbabel.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,23 +24,28 @@
 #include "defs.h"
 
 #if CSVFMTS_ENABLED
+#include <algorithm>               // for for_each, sort
+#include <array>                   // for array
 #include <cctype>                  // for toupper
 #include <cmath>                   // for fabs, floor
-#include <cstdio>                  // for NULL, snprintf, sscanf
-#include <cstdint>
-#include <cstdlib>                 // for atoi, abs, qsort
-#include <cstring>                 // for memset, strstr, strcat, strchr, strlen, strcmp, strcpy, strncpy
-#include <ctime>                   // for gmtime, localtime, strftime
+#include <cstdint>                 // for uint16_t
+#include <cstdio>                  // for sscanf, fprintf, snprintf, stderr
+#include <cstdlib>                 // for abs
+#include <cstring>                 // for strstr, strlen
+#include <ctime>                   // for time_t, gmtime, localtime, strftime
+#include <utility>                 // for pair, make_pair
 
 #include <QByteArray>              // for QByteArray
 #include <QChar>                   // for QChar, QChar::Other_Control
+#include <QDateTime>               // for QDateTime
 #include <QIODevice>               // for QIODevice, QIODevice::ReadOnly, QIODevice::WriteOnly
+#include <QList>                   // for QList, QList<>::const_iterator
 #include <QString>                 // for QString, operator!=
 #include <QStringList>             // for QStringList
 #include <QTextStream>             // for QTextStream
 #include <QVector>                 // for QVector
 #include <Qt>                      // for CaseInsensitive
-#include <QtGlobal>                // for qPrintable
+#include <QtGlobal>                // for qRound, qPrintable
 
 #include "csv_util.h"              // for csv_linesplit
 #include "formspec.h"              // for FormatSpecificDataList
@@ -48,8 +54,8 @@
 #include "inifile.h"               // for inifile_readstr
 #include "jeeps/gpsmath.h"         // for GPS_Math_Known_Datum_To_UTM_EN, GPS_Math_WGS84_To_Known_Datum_M, GPS_Math_WGS84_To_Swiss_EN, GPS_Math_WGS84_To_UKOSMap_M
 #include "src/core/datetime.h"     // for DateTime
+#include "src/core/logging.h"      // for Fatal
 #include "src/core/textstream.h"   // for TextStream
-#include "strptime.h"              // for strptime
 
 
 #define MYNAME "garmin_txt"
@@ -65,7 +71,8 @@ struct gtxt_flags_t {
 
 static gpsbabel::TextStream* fin = nullptr;
 static gpsbabel::TextStream* fout = nullptr;
-static route_head* current_trk, *current_rte;
+static route_head* current_trk;
+static route_head* current_rte;
 static int waypoints;
 static int routepoints;
 static const Waypoint** wpt_a;
@@ -74,10 +81,9 @@ static grid_type grid_index;
 static int datum_index;
 static const char* datum_str;
 static int current_line;
-static char* date_time_format = nullptr;
+static QString date_time_format;
 static int precision = 3;
 static time_t utc_offs = 0;
-
 static gtxt_flags_t gtxt_flags;
 
 enum header_type {
@@ -111,20 +117,17 @@ inline gt_display_modes_e operator++(gt_display_modes_e& s, int) // postfix
   return ret;
 }
 
-#define MAX_HEADER_FIELDS 36
+static std::array<QList<std::pair<QString, int>>, unknown_header> header_mapping_info;
+static QStringList header_column_names;
 
-static char* header_lines[unknown_header + 1][MAX_HEADER_FIELDS];
-static int header_fields[unknown_header + 1][MAX_HEADER_FIELDS];
-static int header_ct[unknown_header + 1];
+static constexpr double kGarminUnknownAlt = 1.0e25;
+static constexpr char kDefaultDateFormat[] = "dd/mm/yyyy";
+static constexpr char kDefaultTimeFormat[] = "HH:mm:ss";
 
-#define GARMIN_UNKNOWN_ALT 1.0e25f
-#define DEFAULT_DISPLAY garmin_display_symbol_and_name
-#define DEFAULT_DATE_FORMAT "dd/mm/yyyy"
-#define DEFAULT_TIME_FORMAT "HH:mm:ss"
-
-/* macros */
-
-#define IS_VALID_ALT(a) (((a) != unknown_alt) && ((a) < GARMIN_UNKNOWN_ALT))
+static bool is_valid_alt(double alt)
+{
+  return (alt != unknown_alt) && (alt < kGarminUnknownAlt);
+}
 
 static char* opt_datum = nullptr;
 static char* opt_dist = nullptr;
@@ -147,31 +150,32 @@ QVector<arglist_t> garmin_txt_args = {
   {"utc",   &opt_utc,         "Write timestamps with offset x to UTC time", nullptr, ARGTYPE_INT, "-23", "+23", nullptr},
 };
 
-struct info_t {
-  double length;
-  time_t start;
-  time_t time;
-  double speed;
-  double total;
-  int count;
-  const Waypoint* prev_wpt;
-  const Waypoint* first_wpt;
-  const Waypoint* last_wpt;
+class PathInfo
+{
+public:
+  double length {0};
+  time_t start {0};
+  time_t time {0};
+  double speed {0};
+  double total {0};
+  int count {0};
+  const Waypoint* prev_wpt {nullptr};
+  const Waypoint* first_wpt {nullptr};
+  const Waypoint* last_wpt {nullptr};
 };
 
-static info_t* route_info;
+static PathInfo* route_info;
 static int route_idx;
-static info_t* cur_info;
+static PathInfo* cur_info;
 
-static const char* headers[] = {
+static const QVector<QString> headers = {
   "Name\tDescription\tType\tPosition\tAltitude\tDepth\tProximity\tTemperature\t"
   "Display Mode\tColor\tSymbol\tFacility\tCity\tState\tCountry\t"
   "Date Modified\tLink\tCategories",
   "Waypoint Name\tDistance\tLeg Length\tCourse",
   "Position\tTime\tAltitude\tDepth\tTemperature\tLeg Length\tLeg Time\tLeg Speed\tLeg Course",
   "Name\tLength\tCourse\tWaypoints\tLink",
-  "Name\tStart Time\tElapsed Time\tLength\tAverage Speed\tLink",
-  nullptr
+  "Name\tStart Time\tElapsed Time\tLength\tAverage Speed\tLink"
 };
 
 /* helpers */
@@ -186,15 +190,16 @@ get_option_val(const char* option, const char* def)
 static void
 init_date_and_time_format()
 {
-  const char* f = get_option_val(opt_date_format, DEFAULT_DATE_FORMAT);
-  date_time_format = convert_human_date_format(f);
+  // This is old, and weird, code.. date_time_format is a global that's
+  // explicitly malloced and freed elsewhere. This isn't very C++ at all,
+  // but this format is on its deathbead for deprecation.
+  const char* d = get_option_val(opt_date_format, kDefaultDateFormat);
+  QString d1 = convert_human_date_format(d);
 
-  date_time_format = xstrappend(date_time_format, " ");
+  const char* t = get_option_val(opt_time_format, kDefaultTimeFormat);
+  QString t1 = convert_human_time_format(t);
 
-  f = get_option_val(opt_time_format, DEFAULT_TIME_FORMAT);
-  const char* c = convert_human_time_format(f);
-  date_time_format = xstrappend(date_time_format, c);
-  xfree((void*) c);
+  date_time_format = QStringLiteral("%1 %2").arg(d1, t1);
 }
 
 static void
@@ -202,7 +207,7 @@ convert_datum(const Waypoint* wpt, double* dest_lat, double* dest_lon)
 {
   double alt;
 
-  if (datum_index == DATUM_WGS84) {
+  if (datum_index == kDautmWGS84) {
     *dest_lat = wpt->latitude;
     *dest_lon = wpt->longitude;
   } else GPS_Math_WGS84_To_Known_Datum_M(wpt->latitude, wpt->longitude, 0.0,
@@ -225,7 +230,7 @@ enum_waypt_cb(const Waypoint* wpt)
     }
     for (int i = 0; i < wpt_a_ct; i++) {		/* check for duplicates */
       const Waypoint* tmp = wpt_a[i];
-      if (case_ignore_strcmp(tmp->shortname, wpt->shortname) == 0) {
+      if (tmp->shortname.compare(wpt->shortname, Qt::CaseInsensitive) == 0) {
         wpt_a[i] = wpt;
         waypoints--;
         return;
@@ -237,24 +242,12 @@ enum_waypt_cb(const Waypoint* wpt)
 
 }
 
-static int
-sort_waypt_cb(const void* a, const void* b)
-{
-  const Waypoint* wa = *(Waypoint**)a;
-  const Waypoint* wb = *(Waypoint**)b;
-  return wa->shortname.compare(wb->shortname, Qt::CaseInsensitive);
-}
-
-
 /* common route and track pre-work */
 
 static void
 prework_hdr_cb(const route_head*)
 {
   cur_info = &route_info[route_idx];
-  cur_info->prev_wpt = nullptr;
-  cur_info->length = 0;
-  cur_info->time = 0;
 }
 
 static void
@@ -365,15 +358,15 @@ print_position(const Waypoint* wpt)
     *fout << "#####\n";
     fatal(MYNAME ": %s (%s) is outside of convertible area \"%s\"!\n",
           wpt->shortname.isEmpty() ? "Waypoint" : qPrintable(wpt->shortname),
-          pretty_deg_format(wpt->latitude, wpt->longitude, 'd', nullptr, 0),
+          qPrintable(pretty_deg_format(wpt->latitude, wpt->longitude, 'd', nullptr, false)),
           gt_get_mps_grid_longname(grid_index, MYNAME));
   }
 }
 
 static void
-print_date_and_time(const time_t time, const int time_only)
+print_date_and_time(const time_t time, const bool time_only)
 {
-  struct tm tm;
+  std::tm tm{};
   char tbuf[32];
 
   if (time < 0) {
@@ -391,7 +384,7 @@ print_date_and_time(const time_t time, const int time_only)
     } else {
       tm = *localtime(&time);
     }
-    strftime(tbuf, sizeof(tbuf), date_time_format, &tm);
+    strftime(tbuf, sizeof(tbuf), CSTR(date_time_format), &tm);
     *fout << QString::asprintf("%s ", tbuf);
   }
   *fout << "\t";
@@ -431,13 +424,13 @@ static void
 print_course(const Waypoint* A, const Waypoint* B)		/* seems to be okay */
 {
   if ((A != nullptr) && (B != nullptr) && (A != B)) {
-    int course = si_round(waypt_course(A, B));
+    int course = qRound(waypt_course(A, B));
     *fout << QString::asprintf("%d° true", course);
   }
 }
 
 static void
-print_distance(const double distance, const int no_scale, const int with_tab, const int decis)
+print_distance(const double distance, const bool no_scale, const bool with_tab, const int decis)
 {
   double dist = distance;
 
@@ -451,7 +444,7 @@ print_distance(const double distance, const int no_scale, const int with_tab, co
       if (dist < 100.0) {
         *fout << QString::asprintf("%.1f mi", dist);
       } else {
-        *fout << QString::asprintf("%d mi", si_round(dist));
+        *fout << QString::asprintf("%d mi", qRound(dist));
       }
     }
   } else {
@@ -462,7 +455,7 @@ print_distance(const double distance, const int no_scale, const int with_tab, co
       if (dist < 100.0) {
         *fout << QString::asprintf("%.1f km", dist);
       } else {
-        *fout << QString::asprintf("%d km", si_round(dist));
+        *fout << QString::asprintf("%d km", qRound(dist));
       }
     }
   }
@@ -472,9 +465,9 @@ print_distance(const double distance, const int no_scale, const int with_tab, co
 }
 
 static void
-print_speed(const double* distance, const time_t* time)
+print_speed(const double distance, const time_t time)
 {
-  double dist = *distance;
+  double dist = distance;
   const char* unit;
 
   if (!gtxt_flags.metric) {
@@ -483,11 +476,11 @@ print_speed(const double* distance, const time_t* time)
   } else {
     unit = "kph";
   }
-  int idist = si_round(dist);
+  int idist = qRound(dist);
 
-  if ((*time != 0) && (idist > 0)) {
-    double speed = MPS_TO_KPH(dist / (double)*time);
-    int ispeed = si_round(speed);
+  if ((time != 0) && (idist > 0)) {
+    double speed = MPS_TO_KPH(dist / (double)time);
+    int ispeed = qRound(speed);
 
     if (speed < 0.01) {
       *fout << QString::asprintf("0 %s", unit);
@@ -543,7 +536,7 @@ write_waypt(const Waypoint* wpt)
   }
   const char* dspl_mode = gt_display_mode_names[i];
 
-  unsigned char wpt_class = garmin_fs_t::get_wpt_class(gmsd, 0);
+  int wpt_class = garmin_fs_t::get_wpt_class(gmsd, 0);
   if (wpt_class <= gt_waypt_class_map_line) {
     wpt_type = gt_waypt_class_names[wpt_class];
   } else {
@@ -568,24 +561,24 @@ write_waypt(const Waypoint* wpt)
 
   print_position(wpt);
 
-  if IS_VALID_ALT(wpt->altitude) {
-    print_distance(wpt->altitude, 1, 0, 0);
+  if (is_valid_alt(wpt->altitude)) {
+    print_distance(wpt->altitude, true, false, 0);
   }
   *fout << "\t";
 
-  double x = WAYPT_GET(wpt, depth, unknown_alt);
+  double x = wpt->depth_value_or(unknown_alt);
   if (x != unknown_alt) {
-    print_distance(x, 1, 0, 1);
+    print_distance(x, true, false, 1);
   }
   *fout << "\t";
 
-  x = WAYPT_GET(wpt, proximity, unknown_alt);
+  x = wpt->proximity_value_or(unknown_alt);
   if (x != unknown_alt) {
-    print_distance(x, 0, 0, 0);
+    print_distance(x, false, false, 0);
   }
   *fout << "\t";
 
-  x = WAYPT_GET(wpt, temperature, -999);
+  x = wpt->temperature_value_or(-999);
   if (x != -999) {
     print_temperature(x);
   }
@@ -604,7 +597,7 @@ write_waypt(const Waypoint* wpt)
   print_string("%s\t", garmin_fs_t::get_state(gmsd, ""));
   const char* country = gt_get_icao_country(garmin_fs_t::get_cc(gmsd, ""));
   print_string("%s\t", (country != nullptr) ? country : "");
-  print_date_and_time(wpt->GetCreationTime().toTime_t(), 0);
+  print_date_and_time(wpt->GetCreationTime().toTime_t(), false);
   if (wpt->HasUrlLink()) {
     UrlLink l = wpt->GetUrlLink();
     print_string("%s\t", l.url_);
@@ -622,16 +615,16 @@ route_disp_hdr_cb(const route_head* rte)
   cur_info = &route_info[route_idx];
   cur_info->prev_wpt = nullptr;
   cur_info->total = 0;
-  if (rte->rte_waypt_ct() <= 0) {
+  if (rte->rte_waypt_empty()) {
     return;
   }
 
   if (!gtxt_flags.route_header_written) {
     gtxt_flags.route_header_written = 1;
-    *fout << QString::asprintf("\r\n\r\nHeader\t%s\r\n", headers[route_header]);
+    *fout << QStringLiteral("\r\n\r\nHeader\t%1\r\n").arg(headers[route_header]);
   }
   print_string("\r\nRoute\t%s\t", rte->rte_name);
-  print_distance(cur_info->length, 0, 1, 0);
+  print_distance(cur_info->length, false, true, 0);
   print_course(cur_info->first_wpt, cur_info->last_wpt);
   *fout << QString::asprintf("\t%d waypoints\t", cur_info->count);
   if (rte->rte_urls.HasUrlLink()) {
@@ -639,7 +632,7 @@ route_disp_hdr_cb(const route_head* rte)
   } else {
     print_string("%s\r\n", "");
   }
-  *fout << QString::asprintf("\r\nHeader\t%s\r\n\r\n", headers[rtept_header]);
+  *fout << QStringLiteral("\r\nHeader\t%1\r\n\r\n").arg(headers[rtept_header]);
 }
 
 static void
@@ -658,11 +651,11 @@ route_disp_wpt_cb(const Waypoint* wpt)
   if (prev != nullptr) {
     double dist = waypt_distance_ex(prev, wpt);
     cur_info->total += dist;
-    print_distance(cur_info->total, 0, 1, 0);
-    print_distance(dist, 0, 1, 0);
+    print_distance(cur_info->total, false, true, 0);
+    print_distance(dist, false, true, 0);
     print_course(prev, wpt);
   } else {
-    print_distance(0, 1, 0, 0);
+    print_distance(0, true, false, 0);
   }
 
   *fout << "\r\n";
@@ -676,25 +669,25 @@ track_disp_hdr_cb(const route_head* track)
   cur_info = &route_info[route_idx];
   cur_info->prev_wpt = nullptr;
   cur_info->total = 0;
-  if (track->rte_waypt_ct() <= 0) {
+  if (track->rte_waypt_empty()) {
     return;
   }
 
   if (!gtxt_flags.track_header_written) {
     gtxt_flags.track_header_written = 1;
-    *fout << QString::asprintf("\r\n\r\nHeader\t%s\r\n", headers[track_header]);
+    *fout << QStringLiteral("\r\n\r\nHeader\t%1\r\n").arg(headers[track_header]);
   }
   print_string("\r\nTrack\t%s\t", track->rte_name);
-  print_date_and_time(cur_info->start, 0);
-  print_date_and_time(cur_info->time, 1);
-  print_distance(cur_info->length, 0, 1, 0);
-  print_speed(&cur_info->length, &cur_info->time);
+  print_date_and_time(cur_info->start, false);
+  print_date_and_time(cur_info->time, true);
+  print_distance(cur_info->length, false, true, 0);
+  print_speed(cur_info->length, cur_info->time);
   if (track->rte_urls.HasUrlLink()) {
     print_string("%s", track->rte_urls.GetUrlLink().url_);
   } else {
     print_string("%s", "");
   }
-  *fout << QString::asprintf("\r\n\r\nHeader\t%s\r\n\r\n", headers[trkpt_header]);
+  *fout << QStringLiteral("\r\n\r\nHeader\t%1\r\n\r\n").arg(headers[trkpt_header]);
 }
 
 static void
@@ -713,29 +706,29 @@ track_disp_wpt_cb(const Waypoint* wpt)
   *fout << "Trackpoint\t";
 
   print_position(wpt);
-  print_date_and_time(wpt->GetCreationTime().toTime_t(), 0);
-  if IS_VALID_ALT(wpt->altitude) {
-    print_distance(wpt->altitude, 1, 0, 0);
+  print_date_and_time(wpt->GetCreationTime().toTime_t(), false);
+  if (is_valid_alt(wpt->altitude)) {
+    print_distance(wpt->altitude, true, false, 0);
   }
 
   *fout << "\t";
-  double depth = WAYPT_GET(wpt, depth, unknown_alt);
+  double depth = wpt->depth_value_or(unknown_alt);
   if (depth != unknown_alt) {
-    print_distance(depth, 1, 0, 1);
+    print_distance(depth, true, false, 1);
   }
 
   if (prev != nullptr) {
     *fout << "\t";
     delta = wpt->GetCreationTime().toTime_t() - prev->GetCreationTime().toTime_t();
-    float temp = WAYPT_GET(wpt, temperature, -999);
+    float temp = wpt->temperature_value_or(-999);
     if (temp != -999) {
       print_temperature(temp);
     }
     *fout << "\t";
     dist = waypt_distance_ex(prev, wpt);
-    print_distance(dist, 0, 1, 0);
-    print_date_and_time(delta, 1);
-    print_speed(&dist, &delta);
+    print_distance(dist, false, true, 0);
+    print_date_and_time(delta, true);
+    print_speed(dist, delta);
     print_course(prev, wpt);
   }
   *fout << "\r\n";
@@ -750,16 +743,16 @@ track_disp_wpt_cb(const Waypoint* wpt)
 static void
 garmin_txt_wr_init(const QString& fname)
 {
-  memset(&gtxt_flags, 0, sizeof(gtxt_flags));
+  gtxt_flags = {};
 
   fout = new gpsbabel::TextStream;
-  fout->open(fname, QIODevice::WriteOnly, MYNAME, "Windows-1252");
+  fout->open(fname, QIODevice::WriteOnly, MYNAME, "windows-1252");
 
   gtxt_flags.metric = (toupper(*get_option_val(opt_dist, "m")) == 'M');
   gtxt_flags.celsius = (toupper(*get_option_val(opt_temp, "c")) == 'C');
   init_date_and_time_format();
   if (opt_precision) {
-    precision = atoi(opt_precision);
+    precision = xstrtoi(opt_precision, nullptr, 10);
     if (precision < 0) {
       fatal(MYNAME ": Invalid precision (%s)!", opt_precision);
     }
@@ -784,10 +777,10 @@ garmin_txt_wr_init(const QString& fname)
 
   switch (grid_index) {
   case grid_bng: /* force datum to "Ord Srvy Grt Britn" */
-    datum_index = DATUM_OSGB36;
+    datum_index = kDatumOSGB36;
     break;
   case grid_swiss: /* force datum to WGS84 */
-    datum_index = DATUM_WGS84;
+    datum_index = kDautmWGS84;
     break;
   default:
     datum_index = gt_lookup_datum_index(datum_str, MYNAME);
@@ -797,7 +790,7 @@ garmin_txt_wr_init(const QString& fname)
     if (case_ignore_strcmp(opt_utc, "utc") == 0) {
       utc_offs = 0;
     } else {
-      utc_offs = atoi(opt_utc);
+      utc_offs = xstrtoi(opt_utc, nullptr, 10);
     }
     utc_offs *= (60 * 60);
     gtxt_flags.utc = 1;
@@ -810,7 +803,8 @@ garmin_txt_wr_deinit()
   fout->close();
   delete fout;
   fout = nullptr;
-  xfree(date_time_format);
+  date_time_format.clear();
+  date_time_format.squeeze();
 }
 
 static void
@@ -831,31 +825,34 @@ garmin_txt_write()
 
   if (waypoints > 0) {
     wpt_a_ct = 0;
-    wpt_a = (const Waypoint**)xcalloc(waypoints, sizeof(*wpt_a));
+    wpt_a = new const Waypoint*[waypoints] {};
     waypt_disp_all(enum_waypt_cb);
     route_disp_all(nullptr, nullptr, enum_waypt_cb);
-    qsort(wpt_a, waypoints, sizeof(*wpt_a), sort_waypt_cb);
+    auto sort_waypt_lambda = [](const Waypoint* wa, const Waypoint* wb)->bool {
+      return wa->shortname.compare(wb->shortname, Qt::CaseInsensitive) < 0;
+    };
+    std::sort(wpt_a, wpt_a + waypoints, sort_waypt_lambda);
 
-    *fout << QString::asprintf("Header\t%s\r\n\r\n", headers[waypt_header]);
+    *fout << QStringLiteral("Header\t%1\r\n\r\n").arg(headers[waypt_header]);
     for (int i = 0; i < waypoints; i++) {
-      const Waypoint* wpt = wpt_a[i];
-      write_waypt(wpt);
+      write_waypt(wpt_a[i]);
     }
-    xfree(wpt_a);
+    delete[] wpt_a;
 
     route_idx = 0;
-    route_info = (info_t*) xcalloc(route_count(), sizeof(info_t));
+    route_info = new PathInfo[route_count()];
     routepoints = 0;
     route_disp_all(prework_hdr_cb, prework_tlr_cb, prework_wpt_cb);
     if (routepoints > 0) {
       route_idx = 0;
       route_disp_all(route_disp_hdr_cb, route_disp_tlr_cb, route_disp_wpt_cb);
     }
-    xfree(route_info);
+    delete[] route_info;
+    route_info = nullptr;
   }
 
   route_idx = 0;
-  route_info = (info_t*) xcalloc(track_count(), sizeof(info_t));
+  route_info = new PathInfo[track_count()];
   routepoints = 0;
   track_disp_all(prework_hdr_cb, prework_tlr_cb, prework_wpt_cb);
 
@@ -863,7 +860,7 @@ garmin_txt_write()
     route_idx = 0;
     track_disp_all(track_disp_hdr_cb, track_disp_tlr_cb, track_disp_wpt_cb);
   }
-  xfree(route_info);
+  delete[] route_info;
 }
 
 /* READER *****************************************************************/
@@ -871,47 +868,91 @@ garmin_txt_write()
 /* helpers */
 
 static void
-free_header(const header_type ht)
+free_headers()
 {
-  for (int i = 0; i < MAX_HEADER_FIELDS; i++) {
-    char* c = header_lines[ht][i];
-    if (c != nullptr) {
-      xfree(c);
-      header_lines[ht][i] = nullptr;
+  std::for_each(header_mapping_info.begin(), header_mapping_info.end(),
+                [](auto& list)->void { list.clear(); });
+}
+
+// Super simple attempt to convert strftime/strptime spec to Qt spec.
+// This misses a LOT of cases and vagaries, but the reality is that we
+// see very few date formats here.
+static QString
+strftime_to_timespec(const char* s)
+{
+  QString q;
+  int l = strlen(s);
+  q.reserve(l * 2); // no penalty if our guess is wrong.
+
+  for (int i = 0; i < l; i++) {
+    switch (s[i]) {
+    case '%':
+      if (i < l-1) {
+        switch (s[++i]) {
+        case 'd':
+          q += "dd";
+          continue;
+        case 'm':
+          q += "MM";
+          continue;
+        case 'y':
+          q += "yy";
+          continue;
+        case 'Y':
+          q += "yyyy";
+          continue;
+        case 'H':
+          q += "hh";
+          continue;
+        case 'M':
+          q += "mm";
+          continue;
+        case 'S':
+          q += "ss";
+          continue;
+        case 'A':
+          q += "dddd";
+          continue;
+        case 'a':
+          q += "ddd";
+          continue;
+        case 'B':
+          q += "MMMM";
+          continue;
+        case 'C':
+          q += "yy";
+          continue;
+        case 'D':
+          q += "MM/dd/yyyy";
+          continue;
+        case 'T':
+          q += "hh:mm:ss";
+          continue;
+        case 'F':
+          q += "yyyy-MM-dd";
+          continue;
+        default:
+          q += s[i+1];
+          break;
+        }
+      }
+      break;
+    default:
+      q += s[i];
+      break;
     }
   }
-  header_ct[ht] = 0;
-  memset(header_fields[ht], 0, sizeof(header_fields[ht]));
+  return q;
 }
 
+
 /* data parsers */
 
-static bool
-parse_date_and_time(const QString& str, time_t* value)
+static QDateTime
+parse_date_and_time(const QString& str)
 {
-  struct tm tm;
-
-  memset(&tm, 0, sizeof(tm));
-  QString tstr = str.trimmed();
-  if (tstr.isEmpty()) {
-    return false;
-  }
-
-  const QByteArray ba = tstr.toUtf8();
-  const char* cin = ba.constData();
-  char* cerr = strptime(cin, date_time_format, &tm);
-  if (cerr == nullptr) {
-    cerr = strptime(cin, "%m/%d/%Y %I:%M:%S %p", &tm);
-    if (cerr == nullptr) {
-      fatal(MYNAME ": Invalid date or/and time \"%s\" at line %d!", qPrintable(tstr), current_line);
-    }
-  }
-
-//	printf(MYNAME "_parse_date_and_time: %02d.%02d.%04d, %02d:%02d:%02d\n",
-//		tm.tm_mday, tm.tm_mon+1, tm.tm_year+1900, tm.tm_hour, tm.tm_min, tm.tm_sec);
-
-  *value = mklocaltime(&tm);
-  return true;
+  QString timespec = strftime_to_timespec(CSTR(date_time_format));
+  return QDateTime::fromString(QString(str).trimmed(), timespec);
 }
 
 static uint16_t
@@ -966,17 +1007,9 @@ parse_temperature(const QString& str, double* temperature)
 static void
 parse_header(const QStringList& lineparts)
 {
-  int column = -1;
-
-  free_header(unknown_header);
-
-  for (const auto& str : lineparts) {
-    column++;
-    header_lines[unknown_header][column] = strupper(xstrdup(str));
-    header_ct[unknown_header]++;
-    if (header_ct[unknown_header] >= MAX_HEADER_FIELDS) {
-      break;
-    }
+  header_column_names.clear();
+  for (const auto& name : lineparts) {
+    header_column_names.append(name.toUpper());
   }
 }
 
@@ -1004,48 +1037,36 @@ bind_fields(const header_type ht)
     fatal(MYNAME ": Incomplete or invalid file header!");
   }
 
-  if (header_ct[unknown_header] <= 0) {
+  if (header_column_names.isEmpty()) {
     return;
   }
-  free_header(ht);
+  header_mapping_info[ht].clear();
 
-  /* make a copy of headers[ht], uppercase, replace "\t" with "\0" */
+  /* make a copy of headers[ht], uppercase, split on "\t" */
+  const QStringList altheader = headers.at(ht).toUpper().split('\t');
 
-  int i = strlen(headers[ht]);
-  char* fields = (char*) xmalloc(i + 2);
-  strcpy(fields, headers[ht]);
-  strcat(fields, "\t");
-  char* c = strupper(fields);
-  while ((c = strchr(c, '\t'))) {
-    *c++ = '\0';
-  }
+  int i = -1;
+  for (const auto& name : qAsConst(header_column_names)) {
+    i++;
 
-  for (i = 0; i < header_ct[unknown_header]; i++) {
-    char* name = header_lines[ht][i] = header_lines[unknown_header][i];
-    header_lines[unknown_header][i] = nullptr;
-
-    c = fields;
-    int field_no = 1;
-    while (*c) {
-      if (strcmp(c, name) == 0) {
-        header_fields[ht][i] = field_no;
-#if 0
-        printf("Binding field \"%s\" to internal number %d (%d,%d)\n", name, field_no, ht, i);
-#endif
-        break;
+    int field_idx = altheader.indexOf(name);
+    if (field_idx >= 0) {
+      int field_no = field_idx + 1;
+      header_mapping_info[ht].append(std::make_pair(name, field_no));
+      if (global_opts.debug_level >= 2) {
+        fprintf(stderr, MYNAME ": Binding field \"%s\" to internal number %d (%d,%d)\n", qPrintable(name), field_no, ht, i);
       }
-      field_no++;
-      c = c + strlen(c) + 1;
+    } else {
+      warning(MYNAME ": Field %s not recognized!\n", qPrintable(name));
     }
   }
-  header_ct[unknown_header] = 0;
-  xfree(fields);
+  header_column_names.clear();
 }
 
 static void
 parse_grid(const QStringList& lineparts)
 {
-  if (lineparts.size() < 1) {
+  if (lineparts.empty()) {
     fatal(MYNAME ": Missing grid headline!\n");
   }
 
@@ -1065,7 +1086,7 @@ parse_grid(const QStringList& lineparts)
 static void
 parse_datum(const QStringList& lineparts)
 {
-  if (lineparts.size() < 1) {
+  if (lineparts.empty()) {
     fatal(MYNAME ": Missing GPS datum headline!\n");
   }
 
@@ -1085,10 +1106,13 @@ parse_waypoint(const QStringList& lineparts)
   wpt->fs.FsChainAdd(gmsd);
 
   for (const auto& str : lineparts) {
-    column++;
+    if (++column >= header_mapping_info[waypt_header].size()) {
+      warning(MYNAME ": too many fields in Waypoint record!\n");
+      break;
+    }
     int i;
     double d;
-    int field_no = header_fields[waypt_header][column];
+    const auto& [name, field_no] = header_mapping_info[waypt_header].at(column);
 
     switch (field_no) {
     case  1:
@@ -1120,17 +1144,17 @@ parse_waypoint(const QStringList& lineparts)
       break;
     case  6:
       if (parse_distance(str, &d, 1, MYNAME)) {
-        WAYPT_SET(wpt, depth, d);
+        wpt->set_depth(d);
       }
       break;
     case  7:
       if (parse_distance(str, &d, 1, MYNAME)) {
-        WAYPT_SET(wpt, proximity, d);
+        wpt->set_proximity(d);
       }
       break;
     case  8:
       if (parse_temperature(str, &d)) {
-        WAYPT_SET(wpt, temperature, d);
+        wpt->set_temperature(d);
       }
       break;
     case  9:
@@ -1158,12 +1182,10 @@ parse_waypoint(const QStringList& lineparts)
       garmin_fs_t::set_country(gmsd, str);
       garmin_fs_t::set_cc(gmsd, gt_get_icao_cc(str, wpt->shortname));
       break;
-    case 16: {
-      time_t ct;
-      if (parse_date_and_time(str, &ct)) {
-        wpt->SetCreationTime(ct);
+    case 16:
+      if (QDateTime dt = parse_date_and_time(str); dt.isValid()) {
+        wpt->SetCreationTime(dt);
       }
-    }
     break;
     case 17: {
       wpt->AddUrlLink(str);
@@ -1188,8 +1210,11 @@ parse_route_header(const QStringList& lineparts)
 
   bind_fields(route_header);
   for (const auto& str : lineparts) {
-    column++;
-    int field_no = header_fields[route_header][column];
+    if (++column >= header_mapping_info[route_header].size()) {
+      warning(MYNAME ": too many fields in Route record!\n");
+      break;
+    }
+    const auto& [name, field_no] = header_mapping_info[route_header].at(column);
     switch (field_no) {
     case 1:
       if (!str.isEmpty()) {
@@ -1213,8 +1238,11 @@ parse_track_header(const QStringList& lineparts)
   bind_fields(track_header);
   auto* trk = new route_head;
   for (const auto& str : lineparts) {
-    column++;
-    int field_no = header_fields[track_header][column];
+    if (++column >= header_mapping_info[track_header].size()) {
+      warning(MYNAME ": too many fields in Track record!\n");
+      break;
+    }
+    const auto& [name, field_no] = header_mapping_info[track_header].at(column);
     switch (field_no) {
     case 1:
       if (!str.isEmpty()) {
@@ -1230,6 +1258,7 @@ parse_track_header(const QStringList& lineparts)
   current_trk = trk;
 }
 
+
 static void
 parse_route_waypoint(const QStringList& lineparts)
 {
@@ -1239,8 +1268,11 @@ parse_route_waypoint(const QStringList& lineparts)
   bind_fields(rtept_header);
 
   for (const auto& str : lineparts) {
-    column++;
-    int field_no = header_fields[rtept_header][column];
+    if (++column >= header_mapping_info[rtept_header].size()) {
+      warning(MYNAME ": too many fields in Route Waypoint record!\n");
+      break;
+    }
+    const auto& [name, field_no] = header_mapping_info[rtept_header].at(column);
     switch (field_no) {
     case 1:
       if (str.isEmpty()) {
@@ -1248,7 +1280,7 @@ parse_route_waypoint(const QStringList& lineparts)
       }
       wpt = find_waypt_by_name(str);
       if (wpt == nullptr) {
-        fatal(MYNAME ": Route waypoint \"%s\" not in waypoint list (line %d)!\n", qPrintable(str), current_line);
+        fatal(FatalMsg() << MYNAME << ": Route waypoint " << str << " not in waypoint list (line " << current_line<< ")!\n");
       }
       wpt = new Waypoint(*wpt);
       break;
@@ -1268,25 +1300,27 @@ parse_track_waypoint(const QStringList& lineparts)
   auto* wpt = new Waypoint;
 
   for (const auto& str : lineparts) {
-    column++;
+    if (++column >= header_mapping_info[trkpt_header].size()) {
+      warning(MYNAME ": too many fields in Trackpoint record!\n");
+      break;
+    }
     double x;
 
     if (str.isEmpty()) {
       continue;
     }
 
-    int field_no = header_fields[trkpt_header][column];
+    const auto& [name, field_no] = header_mapping_info[trkpt_header].at(column);
+
     switch (field_no) {
     case 1:
       parse_coordinates(str, datum_index, grid_index,
                         &wpt->latitude, &wpt->longitude, MYNAME);
       break;
-    case 2: {
-      time_t ct;
-      if (parse_date_and_time(str, &ct)) {
-        wpt->SetCreationTime(ct);
+    case 2:
+      if (QDateTime dt = parse_date_and_time(str); dt.isValid()) {
+        wpt->SetCreationTime(dt);
       }
-    }
     break;
     case 3:
       if (parse_distance(str, &x, 1, MYNAME)) {
@@ -1295,21 +1329,21 @@ parse_track_waypoint(const QStringList& lineparts)
       break;
     case 4:
       if (parse_distance(str, &x, 1, MYNAME)) {
-        WAYPT_SET(wpt, depth, x);
+        wpt->set_depth(x);
       }
       break;
     case 5:
       if (parse_temperature(str, &x)) {
-        WAYPT_SET(wpt, temperature, x);
+        wpt->set_temperature(x);
       }
       break;
     case 8:
       if (parse_speed(str, &x, 1, MYNAME)) {
-        WAYPT_SET(wpt, speed, x);
+        wpt->set_speed(x);
       }
       break;
     case 9:
-      WAYPT_SET(wpt, course, atoi(CSTR(str)));
+      wpt->set_course(xstrtoi(CSTR(str), nullptr, 10));
       break;
     }
   }
@@ -1321,11 +1355,13 @@ parse_track_waypoint(const QStringList& lineparts)
 static void
 garmin_txt_rd_init(const QString& fname)
 {
-  memset(&gtxt_flags, 0, sizeof(gtxt_flags));
+  gtxt_flags = {};
 
   fin = new gpsbabel::TextStream;
-  fin->open(fname, QIODevice::ReadOnly, MYNAME, "Windows-1252");
-  memset(&header_ct, 0, sizeof(header_ct));
+  fin->open(fname, QIODevice::ReadOnly, MYNAME, "windows-1252");
+
+  free_headers();
+  header_column_names.clear();
 
   datum_index = -1;
   grid_index = (grid_type) -1;
@@ -1336,13 +1372,13 @@ garmin_txt_rd_init(const QString& fname)
 static void
 garmin_txt_rd_deinit()
 {
-  for (header_type h = waypt_header; h <= unknown_header; ++h) {
-    free_header(h);
-  }
+  free_headers();
+  header_column_names.clear();
   fin->close();
   delete fin;
   fin = nullptr;
-  xfree(date_time_format);
+  date_time_format.clear();
+  date_time_format.squeeze();
 }
 
 static void
@@ -1361,29 +1397,29 @@ garmin_txt_read()
 
     QStringList lineparts = csv_linesplit(buff, "\t", "", 0);
 
-    if (lineparts.size() < 1) {
+    if (lineparts.empty()) {
       continue;
     }
     auto linetype = lineparts.at(0);
     lineparts.removeFirst();
 
-    if (case_ignore_strcmp(linetype, "Header") == 0) {
+    if (linetype.compare(u"Header", Qt::CaseInsensitive) == 0) {
       parse_header(lineparts);
-    } else if (case_ignore_strcmp(linetype, "Grid") == 0) {
+    } else if (linetype.compare(u"Grid", Qt::CaseInsensitive) == 0) {
       parse_grid(lineparts);
-    } else if (case_ignore_strcmp(linetype, "Datum") == 0) {
+    } else if (linetype.compare(u"Datum", Qt::CaseInsensitive) == 0) {
       parse_datum(lineparts);
-    } else if (case_ignore_strcmp(linetype, "Waypoint") == 0) {
+    } else if (linetype.compare(u"Waypoint", Qt::CaseInsensitive) == 0) {
       parse_waypoint(lineparts);
-    } else if (case_ignore_strcmp(linetype, "Route Waypoint") == 0) {
+    } else if (linetype.compare(u"Route Waypoint", Qt::CaseInsensitive) == 0) {
       parse_route_waypoint(lineparts);
-    } else if (case_ignore_strcmp(linetype, "Trackpoint") == 0) {
+    } else if (linetype.compare(u"Trackpoint", Qt::CaseInsensitive) == 0) {
       parse_track_waypoint(lineparts);
-    } else if (case_ignore_strcmp(linetype, "Route") == 0) {
+    } else if (linetype.compare(u"Route", Qt::CaseInsensitive) == 0) {
       parse_route_header(lineparts);
-    } else if (case_ignore_strcmp(linetype, "Track") == 0) {
+    } else if (linetype.compare(u"Track", Qt::CaseInsensitive) == 0) {
       parse_track_header(lineparts);
-    } else if (case_ignore_strcmp(linetype, "Map") == 0) /* do nothing */ ;
+    } else if (linetype.compare(u"Map", Qt::CaseInsensitive) == 0) /* do nothing */ ;
     else {
       fatal(MYNAME ": Unknown identifier (%s) at line %d!\n", qPrintable(linetype), current_line);
     }
@@ -1391,6 +1427,14 @@ garmin_txt_read()
   }
 }
 
+/*
+ * The file encoding is windows-1252.
+ * Conversion between windows-1252 and utf-16 is handled by the stream.
+ * Conversion between utf-16 and utf-8 is handled by this format.
+ * Let main know char strings have already been converted to utf-8
+ * so it doesn't attempt to re-convert any char strings including gmsd data.
+ */
+
 ff_vecs_t garmin_txt_vecs = {
   ff_type_file,
   FF_CAP_RW_ALL,
@@ -1402,16 +1446,7 @@ ff_vecs_t garmin_txt_vecs = {
   garmin_txt_write,
   nullptr,
   &garmin_txt_args,
-  /*
-   * The file encoding is Windows-1252, a.k.a CET_CHARSET_MS_ANSI.
-   * Conversion between Windows-1252 and utf-16 is handled by the stream.
-   * Conversion between utf-16 and utf-8 is handled by this format.
-   * Let main know char strings have already been converted to utf-8
-   * so it doesn't attempt to re-convert any char strings including gmsd data.
-   */
-  CET_CHARSET_UTF8, 0
-  , NULL_POS_OPS,
-  nullptr
+  NULL_POS_OPS
 };
 
 #endif // CSVFMTS_ENABLED
