@@ -520,8 +520,8 @@ GlobalsatSportFormat::track_read()
           }
 
           /*
-           * GPS year: 2000+; struct tm year: 1900+
-           * GPS month: 1-12, struct tm month: 0-11
+           * GPS year: 2000+; std::tm year: 1900+
+           * GPS month: 1-12, std::tm month: 0-11
            */
 
           QDate gpsDate = QDate(header.dateStart.Year+2000, header.dateStart.Month, header.dateStart.Day);
@@ -644,7 +644,7 @@ GlobalsatSportFormat::track_read()
               wpt->longitude = ((int32_t) point.Longitude) / 1000000.0;
               wpt->latitude = ((int32_t) point.Latitude) / 1000000.0;
               wpt->altitude = point.Altitude;
-              wpt->speed = ((double) point.Speed / 100.0) * 1000.0 / 3600.0;
+              wpt->set_speed(((double) point.Speed / 100.0) * 1000.0 / 3600.0);
               wpt->heartrate = point.HeartRate;
               wpt->cadence = point.Cadence;	//TODO convert in any way??
               wpt->power = point.Power;	//TODO convert in any way??
