@@ -19,10 +19,20 @@
 
  */
 
-#include "defs.h"
-#include <QMap>
-#include <cmath>
-#include <cstdio>
+#include "humminbird.h"
+
+#include <QMap>                 // for QMap
+#include <Qt>                   // for CaseInsensitive
+#include <QtGlobal>             // for qRound
+
+#include <cmath>                // for atan, tan, M_PI, log, sinh
+#include <cstdio>               // for snprintf, SEEK_SET
+#include <cstring>              // for strncpy, memcpy, memset
+
+#include "defs.h"               // for Waypoint, be_read32, be_read16, be_write32, fatal, xfree, be_write16, route_head, xcalloc, track_add_wpt, xstrndup
+#include "mkshort.h"            // for MakeShort
+#include "src/core/datetime.h"  // for DateTime
+
 
 #define MYNAME "humminbird"
 
@@ -59,7 +69,7 @@ Still, they're useful in the code as a plain signature.
 
 /* The hwr data format is records-based, and the records are 36 bytes long. */
 
-struct humminbird_waypt_t {
+struct HumminbirdBase::humminbird_waypt_t {
   /* O.K.: the file can also contain routes with a different magic. */
   /* uint32_t signature; */   /* Just for error checking(?) */
   uint16_t num;          /* Always ascending in the file. */
@@ -74,7 +84,7 @@ struct humminbird_waypt_t {
   char     name[WPT_NAME_LEN];
 };
 
-struct humminbird_rte_t {
+struct HumminbirdBase::humminbird_rte_t {
   /* O.K.: the file can contain also routes with a different magic. */
   /* uint32_t signature; */   /* Just for error checking(?) */
   uint16_t num;
@@ -88,7 +98,7 @@ struct humminbird_rte_t {
   uint16_t points[MAX_RTE_POINTS];
 };
 
-struct humminbird_trk_header_t {      /* 68 bytes, incl signature */
+struct HumminbirdBase::humminbird_trk_header_t {      /* 68 bytes, incl signature */
   /* uint32_t signature; */
   uint16_t trk_num;
   uint16_t zero;
@@ -110,13 +120,13 @@ struct humminbird_trk_header_t {      /* 68 bytes, incl signature */
 };
 
 
-struct humminbird_trk_point_t {
+struct HumminbirdBase::humminbird_trk_point_t {
   int16_t  deltaeast;
   int16_t  deltanorth;
   uint16_t depth;		/* in centimeters */
 };
 
-struct humminbird_trk_header_old_t {      /* 16 bytes, incl signature */
+struct HumminbirdBase::humminbird_trk_header_old_t {      /* 16 bytes, incl signature */
   /* uint32_t signature; */
   uint16_t trk_num;
   uint16_t zero;
@@ -131,12 +141,12 @@ struct humminbird_trk_header_old_t {      /* 16 bytes, incl signature */
 
 };
 
-struct humminbird_trk_point_old_t {
+struct HumminbirdBase::humminbird_trk_point_old_t {
   int16_t  deltaeast;
   int16_t  deltanorth;
 };
 
-struct group_header_t {
+struct HumminbirdBase::group_header_t {
   uint8_t status;
   uint8_t icon;
   uint16_t depth;
@@ -148,7 +158,7 @@ struct group_header_t {
   char name[WPT_NAME_LEN];
 };
 
-struct group_body_t {
+struct HumminbirdBase::group_body_t {
   uint8_t status;
   uint8_t icon;
   uint16_t next_idx;
@@ -156,58 +166,13 @@ struct group_body_t {
 };
 
 
-static const char* humminbird_icons[] = {
-  "Normal",       /*  0 */
-  "House",        /*  1 */
-  "Red cross",    /*  2 */
-  "Fish",         /*  3 */
-  "Duck",         /*  4 */
-  "Anchor",       /*  5 */
-  "Buoy",         /*  6 */
-  "Airport",      /*  7 */
-  "Camping",      /*  8 */
-  "Danger",       /*  9 */
-  "Fuel",         /* 10 */
-  "Rock",         /* 11 */
-  "Weed",         /* 12 */
-  "Wreck",        /* 13 */
-  "Phone",        /* 14 */
-  "Coffee",       /* 15 */
-  "Beer",         /* 16 */
-  "Mooring",      /* 17 */
-  "Pier",         /* 18 */
-  "Slip",         /* 19 */
-  "Ramp",         /* 20 */
-  "Circle",       /* 21 */
-  "Diamond",      /* 22 */
-  "Flag",         /* 23 */
-  "Pattern",      /* 24 */
-  "Shower",       /* 25 */
-  "Water tap",    /* 26 */
-  "Tree",         /* 27 */
-  "Recording",    /* 28 */
-  "Snapshot"      /* 29 */
-};
-
-static gbfile* fin_;
-static gbfile* fout_;
-static int waypoint_num;
-static short_handle wptname_sh, rtename_sh, trkname_sh;
-static humminbird_rte_t* humrte;
-static int rte_num_;
-static QMap<QString, Waypoint*> map;
-
-static
-QVector<arglist_t> humminbird_args = {
-};
-
 /* Takes a latitude in degrees,
  * returns a latitude in degrees. */
-static double
-geodetic_to_geocentric_hwr(const double gd_lat)
+double
+HumminbirdBase::geodetic_to_geocentric_hwr(const double gd_lat)
 {
-  const double cos_ae = 0.9966349016452;
-  const double cos2_ae = cos_ae * cos_ae;
+  constexpr double cos_ae = 0.9966349016452;
+  constexpr double cos2_ae = cos_ae * cos_ae;
   const double gdr = gd_lat *M_PI / 180.0;
 
   return atan(cos2_ae * tan(gdr)) * 180.0/M_PI;
@@ -215,19 +180,19 @@ geodetic_to_geocentric_hwr(const double gd_lat)
 
 /* Takes a latitude in degrees,
  * returns a latitude in degrees. */
-static double
-geocentric_to_geodetic_hwr(const double gc_lat)
+double
+HumminbirdBase::geocentric_to_geodetic_hwr(const double gc_lat)
 {
-  const double cos_ae = 0.9966349016452;
-  const double cos2_ae = cos_ae * cos_ae;
+  constexpr double cos_ae = 0.9966349016452;
+  constexpr double cos2_ae = cos_ae * cos_ae;
   const double gcr = gc_lat *M_PI / 180.0;
 
   return atan(tan(gcr)/cos2_ae) * 180.0/M_PI;
 }
 
 /* Takes a projected "north" value, returns latitude in degrees. */
-static double
-gudermannian_i1924(const double x)
+double
+HumminbirdBase::gudermannian_i1924(const double x)
 {
   const double norm_x = x/i1924_equ_axis;
 
@@ -235,8 +200,8 @@ gudermannian_i1924(const double x)
 }
 
 /* Takes latitude in degrees, returns projected "north" value. */
-static double
-inverse_gudermannian_i1924(const double x)
+double
+HumminbirdBase::inverse_gudermannian_i1924(const double x)
 {
   const double x_r = x/180.0 * M_PI;
   const double guder = log(tan(M_PI/4.0 + x_r/2.0));
@@ -248,20 +213,20 @@ inverse_gudermannian_i1924(const double x)
 * %%%        global callbacks called by gpsbabel main process              %%% *
 *******************************************************************************/
 
-static void
-humminbird_rd_init(const QString& fname)
+void
+HumminbirdBase::humminbird_rd_init(const QString& fname)
 {
   fin_ = gbfopen_be(fname, "rb", MYNAME);
 }
 
-static void
-humminbird_rd_deinit()
+void
+HumminbirdBase::humminbird_rd_deinit() const
 {
   gbfclose(fin_);
 }
 
-static void
-humminbird_read_wpt(gbfile* fin)
+void
+HumminbirdBase::humminbird_read_wpt(gbfile* fin)
 {
   humminbird_waypt_t w;
 
@@ -295,7 +260,7 @@ humminbird_read_wpt(gbfile* fin)
   wpt->altitude  = 0.0; /* It's from a fishfinder... */
 
   if (w.depth != 0) {
-    WAYPT_SET(wpt,depth,(double)w.depth / 100.0);
+    wpt->set_depth((double)w.depth / 100.0);
   }
 
   int num_icons = sizeof(humminbird_icons) / sizeof(humminbird_icons[0]);
@@ -327,8 +292,8 @@ humminbird_read_wpt(gbfile* fin)
   map[buff] = wpt;
 }
 
-static void
-humminbird_read_route(gbfile* fin)
+void
+HumminbirdBase::humminbird_read_route(gbfile* fin) const
 {
 
   humminbird_rte_t hrte;
@@ -366,8 +331,8 @@ humminbird_read_route(gbfile* fin)
   }
 }
 
-static void
-humminbird_read_track(gbfile* fin)
+void
+HumminbirdBase::humminbird_read_track(gbfile* fin)
 {
 
   humminbird_trk_header_t th;
@@ -462,7 +427,7 @@ humminbird_read_track(gbfile* fin)
     wpt->altitude  = 0.0;
 
     if (points[i].depth != 0) {
-      WAYPT_SET(wpt,depth,(double)points[i].depth / 100.0);
+      wpt->set_depth((double)points[i].depth / 100.0);
     }
 
     if (i == th.num_points-2 && th.time != 0) {
@@ -477,12 +442,12 @@ humminbird_read_track(gbfile* fin)
   xfree(points);
 }
 
-static void
-humminbird_read_track_old(gbfile* fin)
+void
+HumminbirdBase::humminbird_read_track_old(gbfile* fin)
 {
 
   humminbird_trk_header_old_t th;
-  const int file_len = 8048;
+  constexpr int file_len = 8048;
   char namebuf[TRK_NAME_LEN];
 
 
@@ -584,8 +549,8 @@ humminbird_read_track_old(gbfile* fin)
   xfree(points);
 }
 
-static void
-humminbird_read()
+void
+HumminbirdBase::humminbird_read()
 {
   while (! gbfeof(fin_)) {
     uint32_t signature = gbfgetuint32(fin_);
@@ -613,54 +578,57 @@ humminbird_read()
 
 /************************************************************************************************/
 
-static void
-humminbird_wr_init(const QString& fname)
+void
+HumminbirdBase::humminbird_wr_init(const QString& fname)
 {
   fout_ = gbfopen_be(fname, "wb", MYNAME);
 
-  wptname_sh = mkshort_new_handle();
+  wptname_sh = new MakeShort;
 
-  setshort_length(wptname_sh, WPT_NAME_LEN - 1);
-  setshort_badchars(wptname_sh, BAD_CHARS);
-  setshort_mustupper(wptname_sh, 0);
-  setshort_mustuniq(wptname_sh, 0);
-  setshort_whitespace_ok(wptname_sh, 1);
-  setshort_repeating_whitespace_ok(wptname_sh, 1);
-  setshort_defname(wptname_sh, "WPT");
+  wptname_sh->set_length(WPT_NAME_LEN - 1);
+  wptname_sh->set_badchars(BAD_CHARS);
+  wptname_sh->set_mustupper(false);
+  wptname_sh->set_mustuniq(false);
+  wptname_sh->set_whitespace_ok(true);
+  wptname_sh->set_repeating_whitespace_ok(true);
+  wptname_sh->set_defname("WPT");
 
-  rtename_sh = mkshort_new_handle();
-  setshort_length(rtename_sh, RTE_NAME_LEN - 1);
-  setshort_badchars(rtename_sh, BAD_CHARS);
-  setshort_mustupper(rtename_sh, 0);
-  setshort_mustuniq(rtename_sh, 0);
-  setshort_whitespace_ok(rtename_sh, 1);
-  setshort_repeating_whitespace_ok(rtename_sh, 1);
-  setshort_defname(rtename_sh, "Route");
+  rtename_sh = new MakeShort;
+  rtename_sh->set_length(RTE_NAME_LEN - 1);
+  rtename_sh->set_badchars(BAD_CHARS);
+  rtename_sh->set_mustupper(false);
+  rtename_sh->set_mustuniq(false);
+  rtename_sh->set_whitespace_ok(true);
+  rtename_sh->set_repeating_whitespace_ok(true);
+  rtename_sh->set_defname("Route");
 
-  trkname_sh = mkshort_new_handle();
-  setshort_length(trkname_sh, RTE_NAME_LEN - 1);
-  setshort_badchars(trkname_sh, BAD_CHARS);
-  setshort_mustupper(trkname_sh, 0);
-  setshort_mustuniq(trkname_sh, 0);
-  setshort_whitespace_ok(trkname_sh, 1);
-  setshort_repeating_whitespace_ok(trkname_sh, 1);
-  setshort_defname(trkname_sh, "Track");
+  trkname_sh = new MakeShort;
+  trkname_sh->set_length(RTE_NAME_LEN - 1);
+  trkname_sh->set_badchars(BAD_CHARS);
+  trkname_sh->set_mustupper(false);
+  trkname_sh->set_mustuniq(false);
+  trkname_sh->set_whitespace_ok(true);
+  trkname_sh->set_repeating_whitespace_ok(true);
+  trkname_sh->set_defname("Track");
 
   waypoint_num = 0;
   rte_num_ = 0;
 }
 
-static void
-humminbird_wr_deinit()
+void
+HumminbirdBase::humminbird_wr_deinit()
 {
-  mkshort_del_handle(&wptname_sh);
-  mkshort_del_handle(&rtename_sh);
-  mkshort_del_handle(&trkname_sh);
+  delete wptname_sh;
+  wptname_sh = nullptr;
+  delete rtename_sh;
+  rtename_sh = nullptr;
+  delete trkname_sh;
+  trkname_sh = nullptr;
   gbfclose(fout_);
 }
 
-static void
-humminbird_write_waypoint(const Waypoint* wpt)
+void
+HumminbirdFormat::humminbird_write_waypoint(const Waypoint* wpt)
 {
   humminbird_waypt_t hum;
   int num_icons = sizeof(humminbird_icons) / sizeof(humminbird_icons[0]);
@@ -681,11 +649,7 @@ humminbird_write_waypoint(const Waypoint* wpt)
     if (hum.icon == 255) {	/* no success, no try to find the item in a more comlex name */
       hum.icon = 0;	/* i.e. "Diamond" as part of "Diamond, Green" or "Green Diamond" */
       for (int i = 0; i < num_icons; i++) {
-        char* match;
-        xasprintf(&match, "*%s*", humminbird_icons[i]);
-        int j = wpt->icon_descr.compare(match, Qt::CaseInsensitive);
-        xfree(match);
-        if (j != 0) {
+        if (wpt->icon_descr.contains(humminbird_icons[i], Qt::CaseInsensitive)) {
           hum.icon = i;
           break;
         }
@@ -693,21 +657,21 @@ humminbird_write_waypoint(const Waypoint* wpt)
     }
   }
 
-  hum.depth = si_round(WAYPT_GET(wpt, depth, 0)*100.0);
+  hum.depth = qRound(wpt->depth_value_or(0) * 100.0);
   be_write16(&hum.depth, hum.depth);
 
   be_write32(&hum.time, wpt->GetCreationTime().toTime_t());
 
   double east = wpt->longitude / 180.0 * EAST_SCALE;
-  be_write32(&hum.east, si_round((east)));
+  be_write32(&hum.east, qRound((east)));
 
   double lat = geodetic_to_geocentric_hwr(wpt->latitude);
   double north = inverse_gudermannian_i1924(lat);
-  be_write32(&hum.north, si_round(north));
+  be_write32(&hum.north, qRound(north));
 
   QString name = (global_opts.synthesize_shortnames)
-                   ? mkshort_from_wpt(wptname_sh, wpt)
-                   : mkshort(wptname_sh, wpt->shortname);
+                 ? wptname_sh->mkshort_from_wpt(wpt)
+                 : wptname_sh->mkshort(wpt->shortname);
   memset(&hum.name, 0, sizeof(hum.name));
   memcpy(&hum.name, CSTR(name), name.length());
 
@@ -715,32 +679,25 @@ humminbird_write_waypoint(const Waypoint* wpt)
   gbfwrite(&hum, sizeof(hum), 1, fout_);
 }
 
-static humminbird_trk_header_t* trk_head;
-static humminbird_trk_point_t* trk_points;
-static int32_t last_east;
-static int32_t last_north;
-static uint32_t last_time;
-
-
-static void
-humminbird_track_head(const route_head* trk)
+void
+HumminbirdHTFormat::humminbird_track_head(const route_head* trk)
 {
   int max_points = (131080 - sizeof(uint32_t)- sizeof(humminbird_trk_header_t)) / sizeof(humminbird_trk_point_t);
 
   trk_head = nullptr;
   last_time = 0;
-  if (trk->rte_waypt_ct() > 0) {
+  if (!trk->rte_waypt_empty()) {
     trk_head = (humminbird_trk_header_t*) xcalloc(1, sizeof(humminbird_trk_header_t));
     trk_points = (humminbird_trk_point_t*) xcalloc(max_points, sizeof(humminbird_trk_point_t));
 
-    QString name = mkshort(trkname_sh, trk->rte_name);
+    QString name = trkname_sh->mkshort(trk->rte_name);
     strncpy(trk_head->name, CSTR(name), sizeof(trk_head->name)-1);
     be_write16(&trk_head->trk_num, trk->rte_num);
   }
 }
 
-static void
-humminbird_track_tail(const route_head*)
+void
+HumminbirdHTFormat::humminbird_track_tail(const route_head* /*unused*/)
 {
   int max_points = (131080 - sizeof(uint32_t)- sizeof(humminbird_trk_header_t)) / sizeof(humminbird_trk_point_t);
 
@@ -776,8 +733,8 @@ humminbird_track_tail(const route_head*)
   trk_points = nullptr;
 }
 
-static void
-humminbird_track_cb(const Waypoint* wpt)
+void
+HumminbirdHTFormat::humminbird_track_cb(const Waypoint* wpt)
 {
   if (trk_head == nullptr) {
     return;
@@ -785,9 +742,9 @@ humminbird_track_cb(const Waypoint* wpt)
 
   int i = trk_head->num_points;
 
-  int32_t east = si_round(wpt->longitude / 180.0 * EAST_SCALE);
+  int32_t east = qRound(wpt->longitude / 180.0 * EAST_SCALE);
   double lat = geodetic_to_geocentric_hwr(wpt->latitude);
-  int32_t north = si_round(inverse_gudermannian_i1924(lat));
+  int32_t north = qRound(inverse_gudermannian_i1924(lat));
 
   if (wpt->creation_time.isValid()) {
     last_time = wpt->GetCreationTime().toTime_t();
@@ -812,7 +769,7 @@ humminbird_track_cb(const Waypoint* wpt)
     int j = i-1;
     trk_points[j].deltaeast = east - last_east;
     trk_points[j].deltanorth = north - last_north;
-    trk_points[j].depth = si_round(WAYPT_GET(wpt, depth, 0)*100.0);
+    trk_points[j].depth = qRound(wpt->depth_value_or(0) * 100.0);
 
     /* BE-ify */
     be_write16(&trk_points[j].deltaeast, trk_points[j].deltaeast);
@@ -841,24 +798,32 @@ humminbird_track_cb(const Waypoint* wpt)
 }
 
 
-static void
-humminbird_track_write()
+void
+HumminbirdHTFormat::write()
 {
-
-  track_disp_all(humminbird_track_head, humminbird_track_tail, humminbird_track_cb);
+  auto humminbird_track_head_lambda = [this](const route_head* rte)->void {
+    humminbird_track_head(rte);
+  };
+  auto humminbird_track_tail_lambda = [this](const route_head* rte)->void {
+    humminbird_track_tail(rte);
+  };
+  auto humminbird_track_cb_lambda = [this](const Waypoint* waypointp)->void {
+    humminbird_track_cb(waypointp);
+  };
+  track_disp_all(humminbird_track_head_lambda, humminbird_track_tail_lambda, humminbird_track_cb_lambda);
 }
 
-static void
-humminbird_rte_head(const route_head* rte)
+void
+HumminbirdFormat::humminbird_rte_head(const route_head* rte)
 {
   humrte = nullptr;
-  if (rte->rte_waypt_ct() > 0) {
+  if (!rte->rte_waypt_empty()) {
     humrte = (humminbird_rte_t*) xcalloc(1, sizeof(*humrte));
   }
 }
 
-static void
-humminbird_rte_tail(const route_head* rte)
+void
+HumminbirdFormat::humminbird_rte_tail(const route_head* rte)
 {
   if (humrte == nullptr) {
     return;
@@ -874,7 +839,7 @@ humminbird_rte_tail(const route_head* rte)
     be_write16(&humrte->num, humrte->num);
     be_write32(&humrte->time, humrte->time);
 
-    QString name = mkshort(rtename_sh, rte->rte_name);
+    QString name = rtename_sh->mkshort(rte->rte_name);
     strncpy(humrte->name, CSTR(name), sizeof(humrte->name)-1);
 
     gbfputuint32(RTE_MAGIC, fout_);
@@ -885,8 +850,8 @@ humminbird_rte_tail(const route_head* rte)
   humrte = nullptr;
 }
 
-static void
-humminbird_write_rtept(const Waypoint* wpt)
+void
+HumminbirdFormat::humminbird_write_rtept(const Waypoint* wpt) const
 {
   if (humrte == nullptr) {
     return;
@@ -905,13 +870,13 @@ humminbird_write_rtept(const Waypoint* wpt)
   }
 }
 
-static void
-humminbird_write_waypoint_wrapper(const Waypoint* wpt)
+void
+HumminbirdFormat::humminbird_write_waypoint_wrapper(const Waypoint* wpt)
 {
-  char* key;
   Waypoint* tmpwpt;
 
-  xasprintf(&key, "%s\01%.9f\01%.9f", CSTRc(wpt->shortname), wpt->latitude, wpt->longitude);
+  QString key = QStringLiteral("%1\01%2\01%3").arg(wpt->shortname)
+                .arg(wpt->latitude, 0, 'f', 9).arg(wpt->longitude, 0, 'f', 9);
   if (!(tmpwpt = map[key])) {
     tmpwpt = const_cast<Waypoint*>(wpt);
     map[key] = const_cast<Waypoint*>(wpt);
@@ -922,64 +887,25 @@ humminbird_write_waypoint_wrapper(const Waypoint* wpt)
     tmpwpt = const_cast<Waypoint*>(wpt);
     tmpwpt->extra_data = p;
   }
-
-  xfree(key);
 }
 
-static void
-humminbird_write()
+void
+HumminbirdFormat::write()
 {
-  waypt_disp_all(humminbird_write_waypoint_wrapper);
-  route_disp_all(nullptr, nullptr, humminbird_write_waypoint_wrapper);
-  route_disp_all(humminbird_rte_head, humminbird_rte_tail, humminbird_write_rtept);
+  auto humminbird_write_waypoint_wrapper_lambda = [this](const Waypoint* waypointp)->void {
+    humminbird_write_waypoint_wrapper(waypointp);
+  };
+  waypt_disp_all(humminbird_write_waypoint_wrapper_lambda);
+  route_disp_all(nullptr, nullptr, humminbird_write_waypoint_wrapper_lambda);
+
+  auto humminbird_rte_head_lambda = [this](const route_head* rte)->void {
+    humminbird_rte_head(rte);
+  };
+  auto humminbird_rte_tail_lambda = [this](const route_head* rte)->void {
+    humminbird_rte_tail(rte);
+  };
+  auto humminbird_write_rtept_lambda = [this](const Waypoint* waypointp)->void {
+    humminbird_write_rtept(waypointp);
+  };
+  route_disp_all(humminbird_rte_head_lambda, humminbird_rte_tail_lambda, humminbird_write_rtept_lambda);
 }
-
-/**************************************************************************/
-
-ff_vecs_t humminbird_vecs = {
-  ff_type_file,
-  {
-    (ff_cap)(ff_cap_read | ff_cap_write) 	/* waypoints */,
-    ff_cap_read 			/* tracks */,
-    (ff_cap)(ff_cap_read | ff_cap_write)	/* routes */
-  },
-  humminbird_rd_init,
-  humminbird_wr_init,
-  humminbird_rd_deinit,
-  humminbird_wr_deinit,
-  humminbird_read,
-  humminbird_write,
-  nullptr, // humminbird_exit,
-  &humminbird_args,
-  CET_CHARSET_ASCII, 1			/* ascii is the expected character set */
-  /* currently fixed !!! */
-  , NULL_POS_OPS,
-  nullptr
-};
-
-/**************************************************************************/
-
-/**************************************************************************/
-
-ff_vecs_t humminbird_ht_vecs = {
-  ff_type_file,
-  {
-    ff_cap_read		 	/* waypoints */,
-    (ff_cap)(ff_cap_read | ff_cap_write)	/* tracks */,
-    ff_cap_read			/* routes */
-  },
-  humminbird_rd_init,
-  humminbird_wr_init,
-  humminbird_rd_deinit,
-  humminbird_wr_deinit,
-  humminbird_read,
-  humminbird_track_write,
-  nullptr, // humminbird_exit,
-  &humminbird_args,
-  CET_CHARSET_ASCII, 1			/* ascii is the expected character set */
-  /* currently fixed !!! */
-  , NULL_POS_OPS,
-  nullptr
-};
-
-/**************************************************************************/
