@@ -87,8 +87,8 @@
 #ifndef LOWRANCEUSR_H_INCLUDED_
 #define LOWRANCEUSR_H_INCLUDED_
 
-#include <cmath>                  // for M_PI, round, atan, exp, log, tan
-#include <ctime>                  // for time_t
+#include <cmath>                // for M_PI, round, atan, exp, log, tan
+#include <cstdint>              // for int64_t
 
 #include <QList>                // for QList
 #include <QString>              // for QString
@@ -101,6 +101,7 @@
 #include "format.h"
 #include "formspec.h"             // for FsChainFind, FsChainAdd, kFsLowranceusr4, FormatSpecificData
 #include "gbfile.h"               // for gbfgetint32, gbfputint32, gbfputint16, gbfgetc, gbfgetint16, gbfwrite, gbfputc, gbfeof, gbfgetflt, gbfclose, gbfgetdbl, gbfopen_le, gbfputdbl, gbfputs, gbfile, gbfputflt, gbfread, gbfseek
+#include "mkshort.h"              // for MakeShort
 #include "src/core/datetime.h"    // for DateTime
 
 
@@ -124,16 +125,6 @@ public:
     return FF_CAP_RW_ALL;
   }
 
-  QString get_encode() const override
-  {
-    return CET_CHARSET_ASCII;
-  }
-
-  int get_fixed_encode() const override
-  {
-    return 0;
-  }
-
   void rd_init(const QString& fname) override;
   void read() override;
   void rd_deinit() override;
@@ -390,11 +381,11 @@ private:
   static constexpr double DEGREESTORADIANS = M_PI/180.0;
   static constexpr int MAX_TRAIL_POINTS = 9999;
   static constexpr double UNKNOWN_USR_ALTITUDE = METERS_TO_FEET(-10000); /* -10000ft is how the unit stores unknown */
-  static constexpr time_t base_time_secs = 946706400; /* Jan 1, 2000 00:00:00 */
+  static constexpr int64_t base_time_secs = 946706400; /* Jan 1, 2000 00:00:00 */
 
   /* Member Functions */
 
-  static char same_points(const Waypoint*, const Waypoint*);
+  static bool same_points(const Waypoint*, const Waypoint*);
   void register_waypt(const Waypoint*) const;
   static const Waypoint* lowranceusr4_find_waypt(uint, int, int);
   static const Waypoint* lowranceusr4_find_global_waypt(uint, uint, uint, uint);
@@ -443,7 +434,7 @@ private:
 
   gbfile*        file_in{};
   gbfile*        file_out{};
-  short_handle   mkshort_handle{};
+  MakeShort*     mkshort_handle{};
 
   route_head*    trk_head{};
   route_head*    rte_head{};
@@ -484,7 +475,7 @@ private:
     }
 
     // Didn't find it in table, default to leave it as the number found
-    return QString("icon-%1").arg(icon);
+    return QStringLiteral("icon-%1").arg(icon);
   }
 
   template <typename T>
