@@ -20,18 +20,17 @@
 
  */
 
-#undef TRACKF_DBG
+static constexpr bool TRACKF_DBG = false;
 
+#include <algorithm>                       // for_each, sort, stable_sort
 #include <cassert>                         // for assert
 #include <cmath>                           // for nan
 #include <cstdio>                          // for printf
 #include <cstdlib>                         // for abs
 #include <cstring>                         // for strlen, strchr, strcmp
-#include <ctime>                           // for gmtime, strftime
+#include <ctime>                           // for gmtime, strftime, time_t, tm
 #include <iterator>                        // for next
 
-#include <algorithm>                       // for sort, stable_sort
-
 #include <QByteArray>                      // for QByteArray
 #include <QChar>                           // for QChar
 #include <QDate>                           // for QDate
@@ -75,7 +74,7 @@ qint64 TrackFilter::trackfilter_parse_time_opt(const char* arg)
 {
   qint64 result = 0;
 
-  QRegularExpression re("^([+-]?\\d+)([wdhms])(?:([+-]?\\d+)([wdhms]))?(?:([+-]?\\d+)([wdhms]))?(?:([+-]?\\d+)([wdhms]))?(?:([+-]?\\d+)([wdhms]))?$", QRegularExpression::CaseInsensitiveOption);
+  static const QRegularExpression re(R"(^([+-]?\d+)([wdhmsz])(?:([+-]?\d+)([wdhmsz]))?(?:([+-]?\d+)([wdhmsz]))?(?:([+-]?\d+)([wdhmsz]))?(?:([+-]?\d+)([wdhmsz]))?(?:([+-]?\d+)([wdhmsz]))?$)", QRegularExpression::CaseInsensitiveOption);
   assert(re.isValid());
   QRegularExpressionMatch match = re.match(arg);
   if (match.hasMatch()) {
@@ -89,18 +88,21 @@ qint64 TrackFilter::trackfilter_parse_time_opt(const char* arg)
 
       switch (match.captured(idx+1).at(0).toLower().toLatin1()) {
       case 'w':
-        partial *= SECONDS_PER_DAY * 7;
+        partial *= SECONDS_PER_DAY * 1000 * 7;
         break;
       case 'd':
-        partial *= SECONDS_PER_DAY;
+        partial *= SECONDS_PER_DAY * 1000;
         break;
       case 'h':
-        partial *= SECONDS_PER_HOUR;
+        partial *= SECONDS_PER_HOUR * 1000;
         break;
       case 'm':
-        partial *= 60;
+        partial *= 60 * 1000;
         break;
       case 's':
+        partial *= 1000;
+        break;
+      case 'z':
         break;
       default:
         fatal(MYNAME "-time: invalid unit in move option \"%s\"!\n", qPrintable(match.captured(idx+1)));
@@ -109,9 +111,10 @@ qint64 TrackFilter::trackfilter_parse_time_opt(const char* arg)
       result += partial;
 
     }
-#ifdef TRACKF_DBG
-    qDebug() << MYNAME "-time option: shift =" << result << "seconds";
-#endif
+
+    if constexpr(TRACKF_DBG) {
+      qDebug() << MYNAME "-time option: shift =" << result / 1000.0 << "seconds";
+    }
   } else {
     fatal(MYNAME "-time: invalid value in move option \"%s\"!\n", arg);
   }
@@ -161,7 +164,7 @@ fix_type TrackFilter::trackfilter_parse_fix(int* nsats)
 QDateTime TrackFilter::trackfilter_get_first_time(const route_head* track)
 {
   if (track->waypoint_list.empty()) {
-    return QDateTime();
+    return {};
   } else {
     return track->waypoint_list.front()->GetCreationTime();
   }
@@ -170,7 +173,7 @@ QDateTime TrackFilter::trackfilter_get_first_time(const route_head* track)
 QDateTime TrackFilter::trackfilter_get_last_time(const route_head* track)
 {
   if (track->waypoint_list.empty()) {
-    return QDateTime();
+    return {};
   } else {
     return track->waypoint_list.back()->GetCreationTime();
   }
@@ -179,7 +182,7 @@ QDateTime TrackFilter::trackfilter_get_last_time(const route_head* track)
 
 void TrackFilter::trackfilter_fill_track_list_cb(const route_head* track) 	/* callback for track_disp_all */
 {
-  if (track->rte_waypt_ct() == 0) {
+  if (track->rte_waypt_empty()) {
     track_del_head(const_cast<route_head*>(track));
     return;
   }
@@ -191,10 +194,6 @@ void TrackFilter::trackfilter_fill_track_list_cb(const route_head* track) 	/* ca
       fatal(FatalMsg() << "track: name option is an invalid expression.");
     }
     if (!regex.match(track->rte_name).hasMatch()) {
-      foreach (Waypoint* wpt, track->waypoint_list) {
-        track_del_wpt(const_cast<route_head*>(track), wpt);
-        delete wpt;
-      }
       track_del_head(const_cast<route_head*>(track));
       return;
     }
@@ -238,9 +237,9 @@ void TrackFilter::trackfilter_split_init_rte_name(route_head* track, const gpsba
   QString datetimestring;
 
   if (opt_interval != 0) {
-    datetimestring = dt.toUTC().toString("yyyyMMddhhmmss");
+    datetimestring = dt.toUTC().toString(u"yyyyMMddhhmmss");
   } else {
-    datetimestring = dt.toUTC().toString("yyyyMMdd");
+    datetimestring = dt.toUTC().toString(u"yyyyMMdd");
   }
 
   if ((opt_title != nullptr) && (strlen(opt_title) > 0)) {
@@ -248,15 +247,15 @@ void TrackFilter::trackfilter_split_init_rte_name(route_head* track, const gpsba
       // Uggh.  strftime format exposed to user.
 
       time_t time = dt.toTime_t();
-      struct tm tm = *gmtime(&time);
+      std::tm tm = *gmtime(&time);
       char buff[128];
       strftime(buff, sizeof(buff), opt_title, &tm);
       track->rte_name = buff;
     } else {
-      track->rte_name = QString("%1-%2").arg(opt_title, datetimestring);
+      track->rte_name = QStringLiteral("%1-%2").arg(opt_title, datetimestring);
     }
   } else if (!track->rte_name.isEmpty()) {
-    track->rte_name = QString("%1-%2").arg(track->rte_name, datetimestring);
+    track->rte_name = QStringLiteral("%1-%2").arg(track->rte_name, datetimestring);
   } else {
     track->rte_name = datetimestring;
   }
@@ -268,14 +267,14 @@ void TrackFilter::trackfilter_pack_init_rte_name(route_head* track, const gpsbab
     // Uggh.  strftime format exposed to user.
 
     gpsbabel::DateTime dt;
-    if (track->rte_waypt_ct() == 0) {
+    if (track->rte_waypt_empty()) {
       dt = default_time;
     } else {
-      auto* wpt = track->waypoint_list.front();
+      const auto* wpt = track->waypoint_list.front();
       dt = wpt->GetCreationTime();
     }
     time_t t = dt.toTime_t();
-    struct tm tm = *gmtime(&t);
+    std::tm tm = *gmtime(&t);
     char buff[128];
     strftime(buff, sizeof(buff), opt_title, &tm);
     track->rte_name = buff;
@@ -325,11 +324,20 @@ void TrackFilter::trackfilter_pack()
 
     route_head* master = track_list.first();
 
+    if (!master->waypoint_list.empty()) {
+      std::for_each(std::next(master->waypoint_list.cbegin()), master->waypoint_list.cend(),
+                    [](Waypoint* wpt)->void {wpt->wpt_flags.new_trkseg = 0;});
+    }
+
     while (track_list.size() > 1) {
       route_head* curr = track_list.takeAt(1);
 
-      foreach (Waypoint* wpt, curr->waypoint_list) {
-        track_del_wpt(curr, wpt);
+      // Steal all the waypoints
+      WaypointList curr_wpts;
+      track_swap_wpts(curr, curr_wpts);
+      // And add them to the master
+      foreach (Waypoint* wpt, curr_wpts) {
+        wpt->wpt_flags.new_trkseg = 0;
         track_add_wpt(master, wpt);
       }
       track_del_head(curr);
@@ -353,15 +361,16 @@ void TrackFilter::trackfilter_merge()
     auto it = track_list.begin();
     while (it != track_list.end()) { /* put all points into temp buffer */
       route_head* track = *it;
-      foreach (Waypoint* wpt, track->waypoint_list) {
-        track_del_wpt(track, wpt); /* copies any new_trkseg flag forward, and clears new_trkseg flag. */
+      // steal all the wpts
+      WaypointList wpts;
+      track_swap_wpts(track, wpts);
+      // add them to the buff or delete them
+      foreach (Waypoint* wpt, wpts) {
         if (wpt->creation_time.isValid()) {
           // we will put the merged points in one track segment,
           // as it isn't clear how track segments in the original tracks
           // should relate to the merged track.
-          // track_del_wpt cleared new_trkseg flag for wpt.
-          // track_add_wpt will set new_trkseg for the first point
-          // added to a track.
+          wpt->wpt_flags.new_trkseg = 0;
           buff.append(wpt);
         } else {
           delete wpt;
@@ -377,7 +386,7 @@ void TrackFilter::trackfilter_merge()
 
     std::stable_sort(buff.begin(), buff.end(), trackfilter_merge_sort_cb);
 
-    Waypoint* prev = nullptr;
+    const Waypoint* prev = nullptr;
 
     for (auto* wpt : buff) {
       if ((prev == nullptr) || (prev->GetCreationTime() != wpt->GetCreationTime())) {
@@ -388,7 +397,7 @@ void TrackFilter::trackfilter_merge()
       }
     }
 
-    if (master->rte_waypt_ct() == 0) {
+    if (master->rte_waypt_empty()) {
       track_del_head(master);
       track_list.clear();
     }
@@ -412,21 +421,18 @@ void TrackFilter::trackfilter_split()
     fatal(MYNAME "-split: Cannot split more than one track, please pack (or merge) before!\n");
   } else if (!track_list.isEmpty()) {
     route_head* master = track_list.first();
-    int count = master->rte_waypt_ct();
-
-    int i, j;
-    double interval = -1; /* seconds */
-    double distance = -1; /* meters */
-
-    if (count <= 1) {
+    if (master->rte_waypt_ct() <= 1) {
       return;
     }
 
+    double interval = -1; /* seconds */
+    double distance = -1; /* meters */
+
     /* check additional options */
 
     opt_interval = (opt_split && (strlen(opt_split) > 0) && (0 != strcmp(opt_split, TRACKFILTER_SPLIT_OPTION)));
     if (opt_interval != 0) {
-      QRegularExpression re(R"(^([+-]?(?:\d+(?:\.\d*)?|\.\d+))([dhms])$)", QRegularExpression::CaseInsensitiveOption);
+      static const QRegularExpression re(R"(^([+-]?(?:\d+(?:\.\d*)?|\.\d+))([dhms])$)", QRegularExpression::CaseInsensitiveOption);
       assert(re.isValid());
       QRegularExpressionMatch match = re.match(opt_split);
       if (match.hasMatch()) {
@@ -452,9 +458,9 @@ void TrackFilter::trackfilter_split()
           fatal(MYNAME ": invalid time interval unit specified.\n");
         }
 
-#ifdef TRACKF_DBG
-        printf(MYNAME ": interval %f seconds\n", interval);
-#endif
+        if constexpr(TRACKF_DBG) {
+          printf(MYNAME ": interval %f seconds\n", interval);
+        }
       } else {
         fatal(MYNAME ": invalid timer interval specified \"%s\", must be a positive number, followed by 'd' for days, 'h' for hours, 'm' for minutes or 's' for seconds.\n", opt_split);
       }
@@ -462,7 +468,7 @@ void TrackFilter::trackfilter_split()
 
     opt_distance = (opt_sdistance && (strlen(opt_sdistance) > 0) && (0 != strcmp(opt_sdistance, TRACKFILTER_SDIST_OPTION)));
     if (opt_distance != 0) {
-      QRegularExpression re(R"(^([+-]?(?:\d+(?:\.\d*)?|\.\d+))([km])$)", QRegularExpression::CaseInsensitiveOption);
+      static const QRegularExpression re(R"(^([+-]?(?:\d+(?:\.\d*)?|\.\d+))([km])$)", QRegularExpression::CaseInsensitiveOption);
       assert(re.isValid());
       QRegularExpressionMatch match = re.match(opt_sdistance);
       if (match.hasMatch()) {
@@ -483,82 +489,76 @@ void TrackFilter::trackfilter_split()
           fatal(MYNAME ": invalid distance unit specified.\n");
         }
 
-#ifdef TRACKF_DBG
-        printf(MYNAME ": distance %f meters\n", distance);
-#endif
+        if constexpr(TRACKF_DBG) {
+          printf(MYNAME ": distance %f meters\n", distance);
+        }
       } else {
         fatal(MYNAME ": invalid distance specified \"%s\", must be a positive number followed by 'k' for kilometers or 'm' for miles.\n", opt_sdistance);
       }
     }
 
-    QList<Waypoint*> buff;
+    // steal all the waypoints
+    WaypointList buff;
+    track_swap_wpts(master, buff);
+    assert(!buff.empty()); // enforced above
 
-    foreach (Waypoint* wpt, master->waypoint_list) {
-      buff.append(wpt);
-    }
+    trackfilter_split_init_rte_name(master, buff.front()->GetCreationTime());
 
-    trackfilter_split_init_rte_name(master, buff.at(0)->GetCreationTime());
+    route_head* curr = master;	/* will be reset by first new track */
 
-    route_head* curr = nullptr;	/* will be set by first new track */
+    // add the first waypoint to the first track
+    track_add_wpt(curr, buff.front());
+    // and add subsequent waypoints to the first track or a new track
+    for (auto prev_it = buff.cbegin(), it = std::next(buff.cbegin()); it != buff.cend(); ++prev_it, ++it) {
+      const Waypoint* prev_wpt = *prev_it;
+      Waypoint* wpt = *it;
 
-    for (i=0, j=1; j<count; i++, j++) {
       bool new_track_flag;
 
       if ((opt_interval == 0) && (opt_distance == 0)) {
-// FIXME: This whole function needs to be reconsidered for arbitrary time.
-        new_track_flag = buff.at(i)->GetCreationTime().toLocalTime().date() !=
-                         buff.at(j)->GetCreationTime().toLocalTime().date();
-#ifdef TRACKF_DBG
-        if (new_track_flag) {
-          printf(MYNAME ": new day %s\n", qPrintable(buff.at(j)->GetCreationTime().toLocalTime().date().toString(Qt::ISODate)));
+//      FIXME: This whole function needs to be reconsidered for arbitrary time.
+        new_track_flag = prev_wpt->GetCreationTime().toLocalTime().date() !=
+                         wpt->GetCreationTime().toLocalTime().date();
+        if constexpr(TRACKF_DBG) {
+          if (new_track_flag) {
+            printf(MYNAME ": new day %s\n", qPrintable(wpt->GetCreationTime().toLocalTime().date().toString(Qt::ISODate)));
+          }
         }
-#endif
       } else {
         new_track_flag = true;
 
         if (distance > 0) {
-          double rt1 = RAD(buff.at(i)->latitude);
-          double rn1 = RAD(buff.at(i)->longitude);
-          double rt2 = RAD(buff.at(j)->latitude);
-          double rn2 = RAD(buff.at(j)->longitude);
-          double curdist = gcdist(rt1, rn1, rt2, rn2);
-          curdist = radtometers(curdist);
+          double curdist = radtometers(
+                             gcdist(RAD(prev_wpt->latitude), RAD(prev_wpt->longitude),
+                                    RAD(wpt->latitude), RAD(wpt->longitude)));
           if (curdist <= distance) {
             new_track_flag = false;
-          }
-#ifdef TRACKF_DBG
-          else {
+          } else if constexpr(TRACKF_DBG) {
             printf(MYNAME ": sdistance, %g > %g\n", curdist, distance);
           }
-#endif
         }
 
         if (interval > 0) {
-          double tr_interval = 0.001 * buff.at(i)->GetCreationTime().msecsTo(buff.at(j)->GetCreationTime());
+          double tr_interval = 0.001 * prev_wpt->GetCreationTime().msecsTo(wpt->GetCreationTime());
           if (tr_interval <= interval) {
             new_track_flag = false;
-          }
-#ifdef TRACKF_DBG
-          else {
+          } else if constexpr(TRACKF_DBG) {
             printf(MYNAME ": split, %g > %g\n", tr_interval, interval);
           }
-#endif
         }
 
       }
       if (new_track_flag) {
-#ifdef TRACKF_DBG
-        printf(MYNAME ": splitting new track\n");
-#endif
+        if constexpr(TRACKF_DBG) {
+          printf(MYNAME ": splitting new track\n");
+        }
         curr = new route_head;
-        trackfilter_split_init_rte_name(curr, buff.at(j)->GetCreationTime());
+        trackfilter_split_init_rte_name(curr, wpt->GetCreationTime());
         track_add_head(curr);
         track_list.append(curr);
       }
-      if (curr != nullptr) {
-        track_del_wpt(master, buff.at(j));
-        track_add_wpt(curr, buff.at(j));
-      }
+      wpt->wpt_flags.new_trkseg = 0;
+      track_add_wpt(curr, wpt);
     }
   }
 }
@@ -574,11 +574,22 @@ void TrackFilter::trackfilter_move()
     return;
   }
 
+  int timeless_points = 0;
+
   for (auto* track : qAsConst(track_list)) {
     foreach (Waypoint* wpt, track->waypoint_list) {
-      wpt->creation_time = wpt->creation_time.addSecs(delta);
+      if (wpt->creation_time.isValid()) {
+        wpt->creation_time = wpt->creation_time.addMSecs(delta);
+      } else {
+        ++timeless_points;
+      }
     }
   }
+  if (timeless_points > 0) {
+    warning(MYNAME "-move: %d points out of %d total points didn't have "
+            "time information and could not be moved.\n",
+            timeless_points, track_waypt_count());
+  }
 }
 
 /*******************************************************************************
@@ -607,12 +618,10 @@ void TrackFilter::trackfilter_synth()
       }
       if (first) {
         if (opt_course) {
-          // TODO: the course value 0 isn't valid, wouldn't it be better to UNSET course?
-          WAYPT_SET(wpt, course, 0);
+          wpt->reset_course();
         }
         if (opt_speed) {
-          // TODO: the speed value 0 isn't valid, wouldn't it be better to UNSET speed?
-          WAYPT_SET(wpt, speed, 0);
+          wpt->reset_speed();
         }
         first = false;
         last_course_lat = wpt->latitude;
@@ -622,9 +631,9 @@ void TrackFilter::trackfilter_synth()
         last_speed_time = wpt->GetCreationTime();
       } else {
         if (opt_course) {
-          WAYPT_SET(wpt, course, heading_true_degrees(RAD(last_course_lat),
-                    RAD(last_course_lon),RAD(wpt->latitude),
-                    RAD(wpt->longitude)));
+          wpt->set_course(heading_true_degrees(RAD(last_course_lat),
+                                               RAD(last_course_lon),RAD(wpt->latitude),
+                                               RAD(wpt->longitude)));
           last_course_lat = wpt->latitude;
           last_course_lon = wpt->longitude;
         }
@@ -638,17 +647,17 @@ void TrackFilter::trackfilter_synth()
             // Note that points with the same time can occur because the input
             // has truncated times, or because we are truncating times with
             // toTime_t().
-            WAYPT_SET(wpt, speed, radtometers(gcdist(
-                                                RAD(last_speed_lat), RAD(last_speed_lon),
-                                                RAD(wpt->latitude),
-                                                RAD(wpt->longitude))) /
-                      (0.001 * std::abs(last_speed_time.msecsTo(wpt->GetCreationTime())))
-                     );
+            wpt->set_speed(radtometers(gcdist(
+                                         RAD(last_speed_lat), RAD(last_speed_lon),
+                                         RAD(wpt->latitude),
+                                         RAD(wpt->longitude))) /
+                           (0.001 * std::abs(last_speed_time.msecsTo(wpt->GetCreationTime())))
+                          );
             last_speed_lat = wpt->latitude;
             last_speed_lon = wpt->longitude;
             last_speed_time = wpt->GetCreationTime();
           } else {
-            WAYPT_UNSET(wpt, speed);
+            wpt->reset_speed();
           }
         }
       }
@@ -665,22 +674,24 @@ QDateTime TrackFilter::trackfilter_range_check(const char* timestr)
 {
   QDateTime result;
 
-  QRegularExpression re("^(\\d{0,14})$");
+  QString start(timestr);
+  QString fmtstart("00000101000000.000");
+  fmtstart.replace(0, start.size(), start);
+
+  static const QRegularExpression re(R"(^\d{14}\.\d{3}$)");
   assert(re.isValid());
-  QRegularExpressionMatch match = re.match(timestr);
+  QRegularExpressionMatch match = re.match(fmtstart);
   if (match.hasMatch()) {
-    QString start = match.captured(1);
-    QString fmtstart("00000101000000");
-    fmtstart.replace(0, start.size(), start);
-    result = QDateTime::fromString(fmtstart, "yyyyMMddHHmmss");
+    // QTime::fromString zzz expects exactly 3 digits representing milliseconds.
+    result = QDateTime::fromString(match.captured(0), "yyyyMMddHHmmss.zzz");
     result.setTimeSpec(Qt::UTC);
     if (!result.isValid()) {
-      fatal(MYNAME "-range-check: Invalid timestamp \"%s\"!\n", qPrintable(start));
+      fatal(MYNAME "-range-check: Invalid timestamp \"%s\"!\n", timestr);
     }
 
-#ifdef TRACKF_DBG
-    qDebug() << MYNAME "-range-check: " << result;
-#endif
+    if constexpr(TRACKF_DBG) {
+      qDebug() << MYNAME "-range-check: " << result;
+    }
   } else {
     fatal(MYNAME "-range-check: Invalid value for option \"%s\"!\n", timestr);
   }
@@ -719,12 +730,13 @@ void TrackFilter::trackfilter_range()
       }
 
       if (!inside) {
-        track_del_wpt(track, wpt);
-        delete wpt;
+        wpt->wpt_flags.marked_for_deletion = 1;
       }
     }
+    // delete marked wpts
+    track_del_marked_wpts(track);
 
-    if (track->rte_waypt_ct() == 0) {
+    if (track->rte_waypt_empty()) {
       track_del_head(track);
       it = track_list.erase(it);
     } else {
@@ -747,19 +759,24 @@ void TrackFilter::trackfilter_seg2trk()
     QList<route_head*> new_track_list;
     for (auto* src : qAsConst(track_list)) {
       new_track_list.append(src);
-      route_head* dest = nullptr;
+      route_head* dest = src;
       route_head* insert_point = src;
       int trk_seg_num = 1;
       bool first = true;
 
-      foreach (Waypoint* wpt, src->waypoint_list) {
+      // steal all the waypoints from the src
+      WaypointList src_wpts;
+      track_swap_wpts(src, src_wpts);
+
+      // and add them back to the original or a new route_head.
+      foreach (Waypoint* wpt, src_wpts) {
         if (wpt->wpt_flags.new_trkseg && !first) {
 
           dest = new route_head;
           dest->rte_num = src->rte_num;
           /* name in the form TRACKNAME #n */
           if (!src->rte_name.isEmpty()) {
-            dest->rte_name = QString("%1 #%2").arg(src->rte_name).arg(++trk_seg_num);
+            dest->rte_name = QStringLiteral("%1 #%2").arg(src->rte_name).arg(++trk_seg_num);
           }
 
           /* Insert after original track or after last newly
@@ -769,18 +786,7 @@ void TrackFilter::trackfilter_seg2trk()
           new_track_list.append(dest);
         }
 
-        /* If we found a track separator, transfer from original to
-         * new track. We have to reset new_trkseg temporarily to
-         * prevent track_del_wpt() from copying it to the next track
-         * point.
-         */
-        if (dest) {
-          unsigned orig_new_trkseg = wpt->wpt_flags.new_trkseg;
-          wpt->wpt_flags.new_trkseg = 0;
-          track_del_wpt(src, wpt);
-          wpt->wpt_flags.new_trkseg = orig_new_trkseg;
-          track_add_wpt(dest, wpt);
-        }
+        track_add_wpt(dest, wpt);
         first = false;
       }
     }
@@ -800,13 +806,12 @@ void TrackFilter::trackfilter_trk2seg()
     while (track_list.size() > 1) {
       route_head* curr = track_list.takeAt(1);
 
+      // steal all the wpts
+      WaypointList curr_wpts;
+      track_swap_wpts(curr, curr_wpts);
+      // and add them to the master
       bool first = true;
-      foreach (Waypoint* wpt, curr->waypoint_list) {
-
-        unsigned orig_new_trkseg = wpt->wpt_flags.new_trkseg;
-        wpt->wpt_flags.new_trkseg = 0;
-        track_del_wpt(curr, wpt);
-        wpt->wpt_flags.new_trkseg = orig_new_trkseg;
+      foreach (Waypoint* wpt, curr_wpts) {
         track_add_wpt(master, wpt);
         if (first) {
           wpt->wpt_flags.new_trkseg = 1;
@@ -826,7 +831,7 @@ TrackFilter::faketime_t TrackFilter::trackfilter_faketime_check(const char* time
 {
   faketime_t result;
 
-  QRegularExpression re(R"(^(f?)(\d{0,14})(?:\+(\d{1,10}))?$)");
+  static const QRegularExpression re(R"(^(f?)(\d{0,14})(?:\+(\d+(?:\.\d*)?|\.\d+))?$)");
   assert(re.isValid());
   QRegularExpressionMatch match = re.match(timestr);
   if (match.hasMatch()) {
@@ -843,7 +848,7 @@ TrackFilter::faketime_t TrackFilter::trackfilter_faketime_check(const char* time
 
     if (match.capturedLength(3) > 0) {
       bool ok;
-      result.step = match.captured(3).toInt(&ok);
+      result.step = llround(1000.0 * match.captured(3).toDouble(&ok));
       if (!ok) {
         fatal(MYNAME "-faketime-check: Invalid step \"%s\"!\n", qPrintable(match.captured(3)));
       }
@@ -851,9 +856,9 @@ TrackFilter::faketime_t TrackFilter::trackfilter_faketime_check(const char* time
       result.step = 0;
     }
 
-#ifdef TRACKF_DBG
-    qDebug() << MYNAME "-faketime option: force =" << result.force << ", timestamp =" << result.start << ", step =" << result.step;
-#endif
+    if constexpr(TRACKF_DBG) {
+      qDebug() << MYNAME "-faketime option: force =" << result.force << ", timestamp =" << result.start << ", step =" << result.step << "milliseconds";
+    }
   } else {
     fatal(MYNAME "-faketime-check: Invalid value for faketime option \"%s\"!\n", timestr);
   }
@@ -871,7 +876,7 @@ void TrackFilter::trackfilter_faketime()
 
       if (!wpt->creation_time.isValid() || faketime.force) {
         wpt->creation_time = faketime.start;
-        faketime.start = faketime.start.addSecs(faketime.step);
+        faketime.start = faketime.start.addMSecs(faketime.step);
       }
     }
   }
@@ -879,69 +884,61 @@ void TrackFilter::trackfilter_faketime()
 
 bool TrackFilter::trackfilter_points_are_same(const Waypoint* wpta, const Waypoint* wptb)
 {
-  // We use a simpler (non great circle) test for lat/lon here as this
-  // is used for keeping the 'bookends' of non-moving points.
-  //
-  // Latitude spacing is about 27 feet per .00001 degree.
-  // Longitude spacing varies, but the reality is that anything closer
-  // than 27 feet does little but clutter the output.
-  // As this is about the limit of consumer grade GPS, it seems a
-  // reasonable tradeoff.
-
   return
-    std::abs(wpta->latitude - wptb->latitude) < .00001 &&
-    std::abs(wpta->longitude - wptb->longitude) < .00001 &&
+    radtometers(gcdist(RAD(wpta->latitude),
+                       RAD(wpta->longitude),
+                       RAD(wptb->latitude),
+                       RAD(wptb->longitude))) < kDistanceLimit &&
     std::abs(wpta->altitude - wptb->altitude) < 20 &&
-    (WAYPT_HAS(wpta,course) == WAYPT_HAS(wptb,course)) &&
-    (wpta->course == wptb->course) &&
-    (wpta->speed == wptb->speed) &&
+    wpta->courses_equal(*wptb) &&
+    wpta->speeds_equal(*wptb) &&
     (wpta->heartrate == wptb->heartrate) &&
     (wpta->cadence == wptb->cadence) &&
-    (wpta->temperature == wptb->temperature);
+    wpta->temperatures_equal(*wptb);
 }
 
 void TrackFilter::trackfilter_segment_head(const route_head* rte)
 {
   double avg_dist = 0;
-  int index = 0;
-  Waypoint* prev_wpt = nullptr;
-  // Consider tossing trackpoints closer than this in radians.
-  // (Empirically determined; It's a few dozen feet.)
-  const double ktoo_close = 0.000005;
+  const Waypoint* prev_wpt = nullptr;
 
-  for (auto it = rte->waypoint_list.cbegin(); it != rte->waypoint_list.cend(); ++it) {
+  const auto wptlist = rte->waypoint_list;
+  for (auto it = wptlist.cbegin(); it != wptlist.cend(); ++it) {
     auto* wpt = *it;
-    if (index > 0) {
-      double cur_dist = gcdist(RAD(prev_wpt->latitude),
-                               RAD(prev_wpt->longitude),
-                               RAD(wpt->latitude),
-                               RAD(wpt->longitude));
-      // Denoise points that are on top of each other.
+    if (it != wptlist.cbegin()) {
+      double cur_dist = radtometers(gcdist(RAD(prev_wpt->latitude),
+                                           RAD(prev_wpt->longitude),
+                                           RAD(wpt->latitude),
+                                           RAD(wpt->longitude)));
+
+      // Denoise points that are on top of each other,
+      // keeping the first and last of the group.
+      if (cur_dist < kDistanceLimit) {
+        if (auto next_it = std::next(it); next_it != wptlist.cend()) {
+          const auto* next_wpt = *next_it;
+          if (trackfilter_points_are_same(prev_wpt, wpt) &&
+              trackfilter_points_are_same(wpt, next_wpt)) {
+            wpt->wpt_flags.marked_for_deletion = 1;
+            continue; // without updating prev_wpt, the first in the group.
+          }
+        }
+      }
+
       if (avg_dist == 0) {
         avg_dist = cur_dist;
       }
 
-      if (cur_dist < ktoo_close) {
-        if (wpt != rte->waypoint_list.back()) {
-          auto* next_wpt = *std::next(it);
-          if (trackfilter_points_are_same(prev_wpt, wpt) &&
-              trackfilter_points_are_same(wpt, next_wpt)) {
-            track_del_wpt(const_cast<route_head*>(rte), wpt);
-            delete wpt;
-            continue;
-          }
-        }
-      }
-      if (cur_dist > .001 && cur_dist > 1.2* avg_dist) {
-        avg_dist = cur_dist = 0;
+      if (cur_dist > 6378.14 && cur_dist > 1.2 * avg_dist) {
+        avg_dist = 0;
         wpt->wpt_flags.new_trkseg = 1;
+      } else {
+        // Update weighted moving average;
+        avg_dist = (cur_dist + 4.0 * avg_dist) / 5.0;
       }
-      // Update weighted moving average;
-      avg_dist = (cur_dist + 4.0 * avg_dist) / 5.0;
     }
     prev_wpt = wpt;
-    index++;
   }
+  track_del_marked_wpts(const_cast<route_head*>(rte));
 }
 
 /*******************************************************************************
@@ -950,7 +947,6 @@ void TrackFilter::trackfilter_segment_head(const route_head* rte)
 
 void TrackFilter::init()
 {
-  RteHdFunctor<TrackFilter> trackfilter_segment_head_f(this, &TrackFilter::trackfilter_segment_head);
   RteHdFunctor<TrackFilter> trackfilter_fill_track_list_cb_f(this, &TrackFilter::trackfilter_fill_track_list_cb);
 
   /*
@@ -962,7 +958,7 @@ void TrackFilter::init()
    */
   need_time = (
                 opt_merge || opt_pack || opt_split || opt_sdistance ||
-                opt_move || opt_fix || opt_speed ||
+                opt_fix || opt_speed ||
                 (trackfilter_opt_count() == 0)	/* do pack by default */
               );
   /* in case of a formatted title we also need valid timestamps */
@@ -972,7 +968,7 @@ void TrackFilter::init()
 
   // Perform segmenting first.
   if (opt_segment) {
-    track_disp_all(trackfilter_segment_head_f, nullptr, nullptr);
+    track_disp_all(trackfilter_segment_head, nullptr, nullptr);
   }
 
   track_list.clear();
